# -*- coding: utf-8 -*-

"""Functions for manipulating FGONG files.  These are provided through
the **FGONG** object and a module function to read an **FGONG** object
from a file.

"""

import numpy as np
import warnings
from .common import integrate, DEFAULT_G
from .adipls import fgong_to_amdl

def load_fgong(filename, N=-1, return_comment=False,
               return_object=False, G=None):
    """Given an FGONG file, returns NumPy arrays `glob` and `var` that
    correspond to the scalar and point-wise variables, as specified
    in the `FGONG format`_.

    .. _FGONG format: https://www.astro.up.pt/corot/ntools/docs/CoRoT_ESTA_Files.pdf

    Also returns the first four lines of the file as a `comment`, if
    desired.

    The version number `ivers` is not implemented.

    If `return_object` is `True`, instead returns an `FGONG` object.
    This will become default behaviour from v0.0.12.  The old
    behaviour will be dropped completely from v0.1.0.

    Parameters
    ----------
    filename: str
        Name of the FGONG file to read.
    N: integer, optional
        Number of characters in each float.  If negative, the function
        tries to guess the size of each float. (default: -1)
    return_comment: bool, optional
        If ``True``, return the first four lines of the FGONG file.
        These are comments that are not used in any calculations.

    Returns
    -------
    glob: NumPy array
        The scalar (or global) variables for the stellar model
    var: NumPy array
        The point-wise variables for the stellar model. i.e. things
        that vary through the star like temperature, density, etc.
    comment: list of strs, optional
        The first four lines of the FGONG file.  These are comments
        that are not used in any calculations.  Only returned if
        ``return_comment=True``.

    """
    with open(filename, 'r') as f:
        comment = [f.readline() for i in range(4)]
        nn, iconst, ivar, ivers = [int(i) for i in f.readline().split()]
        lines = f.readlines()

    tmp = []

    # try to guess the length of each float in the data
    if N < 0: N = len(lines[0])//5

    for line in lines:
        for i in range(len(line)//N):
            s = line[i*N:i*N+N]
            # print(s)
            if s[-9:] == '-Infinity':
                s = '-Inf'
            elif s[-9:] == ' Infinity':
                s = 'Inf'
            elif s.lower().endswith('nan'):
                s = 'nan'
            elif 'd' in s.lower():
                s = s.lower().replace('d','e')

            tmp.append(float(s))

    glob = np.array(tmp[:iconst])
    var = np.array(tmp[iconst:]).reshape((-1, ivar))

    if return_object:
        return FGONG(glob, var, ivers=ivers, G=G,
                     description=comment)
    else:
        warnings.warn("From tomso 0.1.0+, `fgong.load_fgong` will only "
                      "return an `FGONG` object: use `return_object=True` "
                      "to mimic future behaviour",
                      FutureWarning)
        if return_comment:
            return glob, var, comment
        else:
            return glob, var


def save_fgong(filename, glob, var, fmt='%16.9E', ivers=0,
               comment=['\n','\n','\n','\n']):
    """Given data for an FGONG file in the format returned by
    :py:meth:`~tomso.fgong.load_fgong` (i.e. two NumPy arrays and a
    possible header), writes the data to a file.

    This function will be dropped from v0.1.0 in favour of the `to_file`
    function of the `FGONG` object.

    Parameters
    ----------
    filename: str
        Filename to which FGONG data is written.
    glob: NumPy array
        The global variables for the stellar model.
    var: NumPy array
        The point-wise variables for the stellar model. i.e. things
        that vary through the star like temperature, density, etc.
    ivers: int, optional
        The integer indicating the version number of the file.
        (default=0)
    comment: list of strs, optional
        The first four lines of the FGONG file, which usually contain
        notes about the stellar model.

    """
    nn, ivar = var.shape
    iconst = len(glob)

    with open(filename, 'wt') as f:
        f.writelines(comment)

        line = '%10i'*4 % (nn, iconst, ivar, ivers)
        f.writelines([line + '\n'])

        for i in range(0, iconst, 5):
            N = np.mod(i+4, 5)+1  # number of floats in this row
            line = fmt*N % tuple(glob[i:i+5])
            f.writelines([line + '\n'])

        for row in var:
            for i in range(0, ivar, 5):
                N = np.mod(i+4, 5)+1  # number of floats in this row
                line = fmt*N % tuple(row[i:i+5])
                f.writelines([line + '\n'])


def fgong_get(key_or_keys, glob, var, reverse=False, G=DEFAULT_G):
    """Retrieves physical properties of a FGONG model from the ``glob`` and
    ``var`` arrays.

    This function will be dropped from v0.1.0 in favour of the
    attributes of the `FGONG` object.

    Parameters
    ----------
    key_or_keys: str or list of strs
        The desired variable or a list of desired variables.  Current
        options are:

        - ``M``: total mass (float)
        - ``R``: photospheric radius (float)
        - ``L``: total luminosity (float)
        - ``r``: radius (array)
        - ``x``: fractional radius (array)
        - ``m``: mass co-ordinate (array)
        - ``q``: fractional mass co-ordinate (array)
        - ``g``: gravity (array)
        - ``rho``: density (array)
        - ``P``: pressure (array)
        - ``AA``: Ledoux discriminant (array)
        - ``Hp``: pressure scale height (array)
        - ``Hrho``: density scale height (array)
        - ``G1``: first adiabatic index (array)
        - ``T``: temperature (array)
        - ``X``: hydrogen abundance (array)
        - ``L_r``: luminosity at radius ``r`` (array)
        - ``kappa``: opacity (array)
        - ``epsilon``: specific energy generation rate (array)
        - ``cp``: specific heat capacity (array)
        - ``cs2``: sound speed squared (array)
        - ``cs``: sound speed (array)
        - ``tau``: acoustic depth (array)

        For example, if ``glob`` and ``var`` have been returned from
        :py:meth:`~tomso.fgong.load_fgong`, you could use

        >>> M, m = fgong.fgong_get(['M', 'm'], glob, var)

        to get the total mass and mass co-ordinate.  If you only want
        one variable, you don't need to use a list.  The return type
        is just the one corresponding float or array.  So, to get a
        single variable you could use either

        >>> x, = fgong.fgong_get(['x'], glob, var)

        or

        >>> x = fgong.fgong_get('x', glob, var)

    glob: NumPy array
        The scalar (or global) variables for the stellar model
    var: NumPy array
        The point-wise variables for the stellar model. i.e. things
        that vary through the star like temperature, density, etc.
    reverse: bool (optional)
        If ``True``, reverse the arrays so that the first element is
        the centre.
    G: float (optional)
        Value of the gravitational constant.

    Returns
    -------
    output: list of floats and arrays
        A list returning the floats or arrays in the order requested
        by the parameter ``keys``.

    """
    M, R, L = glob[:3]
    r, lnq, T, P, rho, X, L_r, kappa, epsilon, G1 = var[:,:10].T
    cp = var[:,12]
    AA = var[:,14]

    x = r/R
    q = np.exp(lnq)
    m = q*M
    g = G*m/r**2
    Hp = P/(rho*g)
    Hrho = 1/(1/G1/Hp + AA/r)
    cs2 = G1*P/rho                    # square of the sound speed
    cs = np.sqrt(cs2)
    tau = -integrate(1./cs[::-1], r[::-1])[::-1]      # acoustic depth

    if type(key_or_keys) == str:
        keys = [key_or_keys]
        just_one = True
    else:
        keys = key_or_keys
        just_one = False

    I = np.arange(len(var), dtype=int)
    if reverse:
        I = I[::-1]

    output = []
    for key in keys:
        if key == 'M': output.append(M)
        elif key == 'R': output.append(R)
        elif key == 'L': output.append(L)
        elif key == 'r': output.append(r[I])
        elif key == 'x': output.append(x[I])
        elif key == 'm': output.append(m[I])
        elif key == 'q': output.append(q[I])
        elif key == 'g': output.append(g[I])
        elif key == 'rho': output.append(rho[I])
        elif key == 'P': output.append(P[I])
        elif key == 'AA': output.append(AA[I])
        elif key == 'Hp': output.append(Hp[I])
        elif key == 'Hrho': output.append(Hrho[I])
        elif key == 'G1': output.append(G1[I])
        elif key == 'T': output.append(T[I])
        elif key == 'X': output.append(X[I])
        elif key == 'L_r': output.append(L_r[I])
        elif key == 'kappa': output.append(kappa[I])
        elif key == 'epsilon': output.append(epsilon[I])
        elif key == 'cp': output.append(cp[I])
        elif key == 'cs2': output.append(cs2[I])
        elif key == 'cs': output.append(cs[I])
        elif key == 'tau': output.append(tau[I])
        else: raise ValueError('%s is not a valid key for fgong.fgong_get' % key)

    if just_one:
        assert(len(output) == 1)
        return output[0]
    else:
        return output


class FGONG(object):
    """A class that contains and allows one to manipulate the data in a
    stellar model stored in the `FGONG format`_.

    .. _FGONG format: https://www.astro.up.pt/corot/ntools/docs/CoRoT_ESTA_Files.pdf

    The main attributes are the **glob** and **var** arrays, which
    follow the definitions in the FGONG standard.  The data in these
    arrays can be accessed via the attributes with more
    physically-meaningful names (e.g. the radius is ``FGONG.r``).

    Some of these values can also be set via the attributes if doing
    so is unambiguous. For example, the fractional radius **x** is not a
    member of the **var** array but setting **x** will assign the actual
    radius **r**, which is the first column of **var**.  Values that are
    settable are indicated in the list of parameters.

    Parameters
    ----------
    glob: NumPy array
        The global variables for the stellar model.
    var: NumPy array
        The point-wise variables for the stellar model. i.e. things
        that vary through the star like temperature, density, etc.
    ivers: int, optional
        The integer indicating the version number of the file.
        (default=0)
    G: float, optional
        Value for the gravitational constant.  If not given (which is
        the default behaviour), we use ``glob[14]`` if it exists and
        is close to the module-wise default value.  Otherwise, we use
        the module-wise default value.
    description: list of 4 strs, optional
        The first four lines of the FGONG file, which usually contain
        notes about the stellar model.

    Attributes
    ----------
    iconst: int
        number of global data entries (i.e. length of **glob**)
    nn: int
        number of points in stellar model (i.e. number of rows in **var**)
    ivar: int
        number of variables recorded at each point in stellar model
        (i.e. number of columns in **var**)
    M: float, settable
        total mass
    R: float, settable
        photospheric radius
    L: float, settable
        total luminosity
    r: NumPy array, settable
        radius co-ordinate
    lnq: NumPy array, settable
        natural logarithm of the fractional mass co-ordinate
    T: NumPy array, settable
        temperature
    P: NumPy array, settable
        pressure
    rho: NumPy array, settable
        density
    X: NumPy array, settable
        fractional hydrogen abundance (by mass)
    L_r: NumPy array, settable
        luminosity at radius **r**
    kappa: NumPy array, settable
        Rosseland mean opacity
    epsilon: NumPy array, settable
        specific energy generation rate
    Gamma_1: NumPy array, settable
        first adiabatic index, aliased by **G1**
    G1: NumPy array, settable
        first adiabatic index, alias of **Gamma_1**
    cp: NumPy array, settable
        specific heat capacity
    AA: NumPy array, settable
        Ledoux discriminant
    Z: NumPy array, settable
        metal abundance
    x: NumPy array, settable
        fractional radius co-ordinate
    q: NumPy array, settable
        fractional mass co-ordinate
    m: NumPy array, settable
        mass co-ordinate
    g: NumPy array
        local gravitational acceleration
    Hp: NumPy array
        pressure scale height
    Hrho: NumPy array
        density scale height
    N2: NumPy array
        squared Brunt–Väisälä (angular) frequency
    cs2: NumPy array
        squared adiabatic sound speed
    cs: NumPy array
        adiabatic sound speed
    U: NumPy array
        homology invariant *dlnm/dlnr*
    V: NumPy array
        homology invariant *dlnP/dlnr*
    Vg: NumPy array
        homology invariant *V/Gamma_1*

    """

    def __init__(self, glob, var, ivers=300, G=None,
                 description=['\n', '\n', '\n', '\n']):
        self.ivers = ivers
        self.glob = glob
        self.var = var
        self.description = description

        # if G is None, use glob[14] if it exists and looks like a
        # reasonable value of G
        if G is None:
            if len(glob) >= 14 and np.isclose(glob[14], DEFAULT_G,
                                              rtol=1e-3, atol=0.01e-8):
                self.G = glob[14]
            else:
                self.G = DEFAULT_G
        else:
            self.G = G

    def __len__(self):
        return len(self.var)

    def to_file(self, filename, fmt='%16.9E'):
        """Save the model to an FGONG file.

        Parameters
        ----------
        filename: str
            Filename to which the data is written.
        fmt: str, optional
            Format string for floating point numbers in the **glob**
            and **var** arrays.
        """
        save_fgong(filename, self.glob, self.var, fmt=fmt,
                   ivers=self.ivers, comment=self.description)

    def to_amdl(self):
        """Convert the model to an ``ADIPLSStellarModel`` object."""
        from .adipls import ADIPLSStellarModel

        return ADIPLSStellarModel(
            *fgong_to_amdl(self.glob, self.var, G=self.G), G=self.G)

    def to_gyre(self, version=None):
        """Convert the model to a ``GYREStellarModel`` object."""
        from .gyre import gyre_header_dtypes, gyre_data_dtypes, GYREStellarModel

        if version is None:
            version = max([k for k in gyre_header_dtypes.keys()])

        header = np.zeros(1, gyre_header_dtypes[version])
        header['M'] = self.glob[0]
        header['R'] = self.glob[1]
        header['L'] = self.glob[2]

        if version > 1:
            header['version'] = version

        data = np.zeros(self.nn, gyre_data_dtypes[version])
        # data['r'] = self.var[:,0]
        # data['T'] = self.var[:,2]
        # data['P'] = self.var[:,3]
        # data['rho'] = self.var[:,4]

        # if np.all(np.diff(data['r']) <= 0):
        #     return GYREStellarModel(header, data[::-1], G=self.G)
        # else:
        #     return GYREStellarModel(header, data, G=self.G)

        g = GYREStellarModel(header[0], data, G=self.G)

        g.r = self.r
        g.m = self.m
        g.T = self.T
        g.P = self.P
        g.rho = self.rho
        g.Gamma_1 = self.Gamma_1
        g.N2 = self.N2
        g.kappa = self.kappa
        g.L_r = self.L_r
        g.data['nabla_ad'] = self.var[:,10]
        g.data['delta'] = self.var[:,11]

        # The definitions of epsilon in FGONG and GYRE formats might
        # be different.  Compute non-adiabatic modes at your peril!
        if version < 101:
            g.data['eps_tot'] = self.epsilon
        else:
            g.data['eps'] = self.epsilon

        if np.all(np.diff(g.r) <= 0):
            g.data = g.data[::-1]

        g.data['k'] = np.arange(self.nn) + 1
        return g

    # FGONG parameters that can be derived from data
    @property
    def iconst(self): return len(self.glob)

    @property
    def nn(self): return self.var.shape[0]

    @property
    def ivar(self): return self.var.shape[1]

    # Various properties for easier access to the data in `glob` and
    # `var`.

    @property
    def M(self): return self.glob[0]

    @M.setter
    def M(self, val): self.glob[0] = val

    @property
    def R(self): return self.glob[1]

    @R.setter
    def R(self, val): self.glob[1] = val

    @property
    def L(self): return self.glob[2]

    @L.setter
    def L(self, val): self.glob[2] = val

    @property
    def r(self): return self.var[:,0]

    @r.setter
    def r(self, val):
        self.var[:,0] = val
        self.var[:,17] = self.R-val

    @property
    def lnq(self): return self.var[:,1]

    @lnq.setter
    def lnq(self, val): self.var[:,1] = val

    @property
    def T(self): return self.var[:,2]

    @T.setter
    def T(self, val): self.var[:,2] = val

    @property
    def P(self): return self.var[:,3]

    @P.setter
    def P(self, val): self.var[:,3] = val

    @property
    def rho(self): return self.var[:,4]

    @rho.setter
    def rho(self, val): self.var[:,4] = val

    @property
    def X(self): return self.var[:,5]

    @X.setter
    def X(self, val): self.var[:,5] = val

    @property
    def L_r(self): return self.var[:,6]

    @L_r.setter
    def L_r(self, val): self.var[:,6] = val

    @property
    def kappa(self): return self.var[:,7]

    @kappa.setter
    def kappa(self): self.var[:,7] = val

    @property
    def epsilon(self): return self.var[:,8]

    @epsilon.setter
    def epsilon(self, val): self.var[:,8] = val

    @property
    def Gamma_1(self): return self.var[:,9]

    @Gamma_1.setter
    def Gamma_1(self, val): self.var[:,9] = val

    @property
    def G1(self): return self.var[:,9]

    @G1.setter
    def G1(self, val): self.var[:,9] = val

    @property
    def cp(self): return self.var[:,12]

    @cp.setter
    def cp(self, val): self.var[:,12] = val

    @property
    def AA(self): return self.var[:,14]

    @AA.setter
    def AA(self, val): self.var[:,14] = val

    @property
    def Z(self): return self.var[:,16]

    @Z.setter
    def Z(self, val): self.var[:,16] = val

    # Some convenient quantities derived from `glob` and `var`.
    @property
    def x(self): return self.r/self.R

    @x.setter
    def x(self, val): self.r = val*self.R

    @property
    def q(self): return np.exp(self.lnq)

    @q.setter
    def q(self, val): self.lnq = np.log(val)

    @property
    def m(self): return self.q*self.M

    @m.setter
    def m(self, val): self.q = val/self.M

    @property
    def g(self): return self.G*self.m/self.r**2

    @property
    def Hp(self): return self.P/(self.rho*self.g)

    @property
    def Hrho(self): return 1/(1/self.Gamma_1/self.Hp + self.AA/self.r)

    @property
    def N2(self):
        val = self.AA*self.g/self.r
        val[self.x==0] = 0
        return val

    @property
    def cs2(self): return self.Gamma_1*self.P/self.rho

    @property
    def cs(self): return self.cs2**0.5

    @property
    def U(self):
        val = 4.*np.pi*self.rho*self.r**3/self.m
        val[self.x==0] = 3
        return val

    @property
    def V(self):
        val = self.G*self.m*self.rho/self.P/self.r
        val[self.x==0] = 0
        return val

    @property
    def Vg(self): return self.V/self.Gamma_1

    # - ``G1``: first adiabatic index (array)
    # - ``tau``: acoustic depth (array)



"""
Functions for manipulating `MESA`_ input and output files.

.. _MESA: http://mesa.sourceforge.net/
"""

import numpy as np
import warnings
from tomso.common import tomso_open, load_mesa_gyre


def load_history(filename, prune=False, return_object=False):
    """Reads a MESA history file and returns the global data and history
    data in two structured arrays.  Uses builtin `gzip` module to read
    files ending with `.gz`.

    If `return_object` is `True`, instead returns an `MESALog` object.
    This will become default behaviour from v0.0.12.  The old
    behaviour will be dropped completely from v0.1.0.

    Parameters
    ----------
    filename: str
        Filename of the MESA history file to load.
    prune: bool, optional
        If `True`, make the model number monotonic by only using the
        last model of with any given model number and restrict models
        to those with model number less than that of the last model.
        Useful for removing apparent reversals in time or model number
        because of backups and retries, and for models that finished
        with fewer models following a restart.

    Returns
    -------
    header: structured array
        Global data for the evolutionary run. e.g. initial parameters.
        The keys for the array are the MESA variable names as in
        `history.columns`.

    data: structured array
        History data for the run. e.g. age, effective temperature.
        The keys for the array are the MESA variable names as in
        `history.columns`.

    """

    header, data = load_mesa_gyre(filename, 'mesa')
    if prune:
        data = data[data['model_number'] <= data['model_number'][-1]]
        I = np.unique(data['model_number'][::-1], return_index=True)[1][::-1]
        data = data[len(data) - I - 1][::-1]

    if return_object:
        return MESALog(header, data)
    else:
        warnings.warn("From tomso 0.1.0+, `mesa.load_history` will only "
                      "return a `MESALog` object: use `return_object=True` "
                      "to mimic future behaviour",
                      FutureWarning)
        return header, data


def load_profile(filename, return_object=False):
    """Reads a MESA profile and returns the global data and profile
    data in two structured arrays.  Uses builtin `gzip` module to read
    files ending with `.gz`.

    If `return_object` is `True`, instead returns an `MESALog` object.
    This will become default behaviour from v0.0.12.  The old
    behaviour will be dropped completely from v0.1.0.

    Parameters
    ----------
    filename: str
        Filename of the MESA profile to load.

    Returns
    -------
    header: structured array
        Global data for the stellar model. e.g. total mass, luminosity.
        The keys for the array are the MESA variable names as in
        `profile.columns`.

    data: structured array
        Profile data for the stellar model. e.g. radius, pressure.
        The keys for the array are the MESA variable names as in
        `profile.columns`.
    """

    header, data = load_mesa_gyre(filename, 'mesa')
    if return_object:
        return MESALog(header, data)
    else:
        warnings.warn("From tomso 0.1.0+, `mesa.load_profile` will only "
                      "return a `MESALog` object: use `return_object=True` "
                      "to mimic future behaviour",
                      FutureWarning)
        return header, data


def load_astero_results(filename):
    """Reads a set of MESA results from one of the optimization routines
    in the `astero` module.

    Parameters
    ----------
    filename: str
        Filename of the file containing the results.

    Returns
    -------
    data: structured array
        Array with all the results.
    """
    with tomso_open(filename, 'rb') as f:
        lines = [line.replace(b'D', b'E') for line in f.readlines()]

    # the last column results for `search_type = simplex` fits have a
    # nameless column that says what kind of simplex step was taken.
    # we have to give it a name ourselves
    names = [name.decode('utf-8') for name in lines[1].split()]
    N_columns = len(lines[2].split())
    if len(names) == N_columns - 1:
        names.append('step_type')

    data = np.genfromtxt(lines[2:-4], dtype=None, names=names,
                         encoding='utf-8')

    return data


def load_sample(filename):
    """Reads a MESA sample file that describes a model from one of the
    optimization routines in the `astero` module.

    Parameters
    ----------
    filename: str
        Filename of the file containing the result.

    Returns
    -------
    d: dict
        A dictionary containing all the results.

    """
    with tomso_open(filename, 'rb') as f:
        # lines = [line.split() for line in f.read().decode('utf-8').split('\n')
        #          if line.strip()]
        lines = [line.decode('utf-8').split() for line in f.readlines() if line.strip()]

    table_dtype = [('n', int), ('chi2term', float), ('freq', float), ('corr', float),
                   ('obs', float), ('sigma', float), ('logE', float)]
    d = {'l%i' % ell: np.zeros(0, dtype=table_dtype) for ell in range(4)}
    # d = {'l0': np.zeros(0, dtype=table_dtype),
    #      'l1': np.zeros(0, dtype=table_dtype),
    #      'l2': np.zeros(0, dtype=table_dtype),
    #      'l3': np.zeros(0, dtype=table_dtype)}
    ell = 0

    for line in lines:
        if line[0][:2] == 'l=':
            ell = int(line[0][-1])
        elif len(line) == 7:
            # I'm not quite sure why this hideous construction is
            # necessary but it seems that the recarray construction
            # depends on whether it gets a tuple or a list
            row = np.array(tuple([int(line[0])] + list(map(float, line[1:]))),
                           dtype=table_dtype)
            d['l%i' % ell] = np.append(d['l%i' % ell], row)
            # d['l%i' % ell]['n'].append(int(line[0]))
            # d['l%i' % ell]['chi2'].append(float(line[1]))
            # d['l%i' % ell]['mdl'].append(float(line[2]))
            # d['l%i' % ell]['cor'].append(float(line[3]))
            # d['l%i' % ell]['obs'].append(float(line[4]))
            # d['l%i' % ell]['err'].append(float(line[5]))
            # d['l%i' % ell]['logE'].append(float(line[6]))
        else:
            key = ''.join([word + ' ' for word in line[:-1]])[:-1]
            # if key == 'mass/Msun':
            #     key = 'initial mass'

            value = float(line[-1].lower().replace('d', 'e'))
            d[key] = value

    return d


# update_inlist, string_where and replace_value all ported from
# mesaface.  still need testing!
def update_inlist(inlist, d):
    """Updates parameter values in a MESA inlist file.  The function
    searches the whole file for the parameter key.  An ``IndexError``
    usually means that one of the keys in dict `d` wasn't found in
    `inlist`.

    Parameters
    ----------
    inlist: str
        Filename of the inlist file that will be updated.
    d: dict
        Dictionary containing the parameter names and their new
        values. e.g. `{'initial_mass': 1.0}` or
        `{'use_Ledoux_criterion': True}`.

    """
    with open(inlist, 'r') as f: lines = f.readlines()

    # don't search comments
    search_lines = [line.split('!', 1)[0] for line in lines]

    for key, value in d.items():
        i = string_where(search_lines, key)[0]
        lines[i] = replace_value(lines[i], value)

    with open(inlist, 'wt') as f:
        f.writelines(lines)


def string_where(lines, expr):
    "Returns list of indices of the lines in `lines` containing `expr`."
    return [i for i in range(len(lines)) if expr in lines[i].split()]


def replace_value(line, value):
    """Replaces the parameter `value` in the given `line` of a MESA
    inlist.  Format is inferred from the type of value: `float`,
    `str`, `int` or `bool`.

    """
    equals = line.index('=')+1
    if type(value) == float:
        return '%s %.20e\n' % (line[:equals], value)
    elif type(value) == str:
        return '%s %s\n' % (line[:equals], value)
    elif type(value) == int:
        return '%s %i\n' % (line[:equals], value)
    elif type(value) == bool:
        if value:
            return '%s .true.\n' % line[:equals]
        else:
            return '%s .false.\n' % line[:equals]
    else:
        raise ValueError('Value in mesa.replace_value() is not a valid type!')


class MESALog(object):
    def __init__(self, header, data):
        self.header = header
        self.data = data

    def __len__(self):
        return len(self.data)

    def __getitem__(self, key):
        if type(key) in (int, slice):
            return self.data[key]
        elif type(key) is str:
            for source in [self.data, self.header]:
                names = source.dtype.names
                if key in names:
                    return source[key]
                elif ('log_' + key) in names:
                    return 10.**source['log_' + key]
                elif ('log' + key) in names:
                    return 10.**source['log' + key]
                elif key.startswith('log_') and key[4:] in names:
                    return np.log10(source[key[4:]])
                elif key.startswith('log') and key[3:] in names:
                    return np.log10(source[key[3:]])



from tomso import fgong
import numpy as np
import unittest

tmpfile = 'data/tmpfile'
EPS = np.finfo(float).eps

class TestFGONGFunctions(unittest.TestCase):

    def test_load_fgong(self):
        glob, var, comment = fgong.load_fgong('data/modelS.fgong', return_comment=True)
        self.assertEqual(comment[0][:6], 'L5BI.D')
        self.assertEqual(len(glob), 15)
        self.assertEqual(len(var), 2482)
        self.assertEqual(len(var[0]), 30)

        self.assertAlmostEqual(glob[0], 1.989e33)
        self.assertAlmostEqual(glob[1], 6.959906258e10)
        self.assertAlmostEqual(glob[2], 3.845999350e33)

        m = fgong.load_fgong('data/modelS.fgong', return_object=True)
        self.assertEqual(glob[0], m.M)
        self.assertEqual(glob[1], m.R)
        self.assertEqual(glob[2], m.L)
        self.assertTrue(np.all(m.m == np.exp(var[:,1])*glob[0]))
        self.assertTrue(np.all(m.x == var[:,0]/glob[1]))
        self.assertTrue(np.all(m.G1 == m.Gamma_1))

    def test_save_fgong(self):
        glob1, var1, comment1 = fgong.load_fgong('data/modelS.fgong', return_comment=True)
        fgong.save_fgong(tmpfile, glob1, var1, comment=comment1, fmt='%16.9E')
        glob2, var2, comment2 = fgong.load_fgong(tmpfile, return_comment=True)
        for i in range(len(glob1)):
            self.assertAlmostEqual(glob1[i], glob2[i])

        for line1, line2 in zip(comment1, comment2):
            self.assertEqual(line1, line2)

        for i in range(len(var1)):
            for j in range(len(var1[i])):
                self.assertAlmostEqual(var1[i,j], var2[i,j])

        m1 = fgong.load_fgong('data/modelS.fgong', return_object=True)
        m1.to_file(tmpfile, fmt='%16.9E')
        m2 = fgong.load_fgong(tmpfile, return_object=True)

        for i in range(m1.iconst):
            self.assertAlmostEqual(m1.glob[i], m2.glob[i])

        for line1, line2 in zip(m1.description, m2.description):
            self.assertEqual(line1, line2)

        for i in range(m1.nn):
            for j in range(m1.ivar):
                self.assertAlmostEqual(m1.var[i,j], m2.var[i,j])

    def test_fgong_get(self):
        glob, var = fgong.load_fgong('data/modelS.fgong')
        x, r, R, M = fgong.fgong_get(['x','r','R','M'], glob, var)
        self.assertTrue(np.all(x==r/R))
        self.assertAlmostEqual(M, 1.989e33)
        self.assertAlmostEqual(R, 6.959906258e10)

        cs, = fgong.fgong_get(['cs'], glob, var)
        cs2 = fgong.fgong_get('cs2', glob, var)
        self.assertTrue(np.allclose(cs**2, cs2))

        m = fgong.load_fgong('data/modelS.fgong', return_object=True)
        self.assertTrue(np.all(m.x==m.r/m.R))
        self.assertAlmostEqual(m.M, 1.989e33)
        self.assertAlmostEqual(m.R, 6.959906258e10)
        self.assertTrue(np.allclose(m.cs**2, m.cs2))

    def test_fgong_get_reverse(self):
        glob, var = fgong.load_fgong('data/modelS.fgong')
        x_fwd, = fgong.fgong_get(['x'], glob, var)
        x_rev = fgong.fgong_get('x', glob, var, reverse=True)
        self.assertTrue(np.all(x_fwd[::-1] == x_rev))

    def test_fgong_get_cross_check(self):
        glob, var = fgong.load_fgong('data/modelS.fgong')
        M, R, L, r, x, m, q, g, rho, P, Hp, G1, T, X, L_r, kappa, epsilon, cs2, cs, tau \
            = fgong.fgong_get(['M','R', 'L', 'r', 'x', 'm', 'q', 'g',
                               'rho', 'P', 'Hp', 'G1', 'T', 'X', 'L_r',
                               'kappa', 'epsilon', 'cs2', 'cs', 'tau'],
                           glob, var)
        self.assertTrue(np.allclose(q, m/M, rtol=4*EPS))
        self.assertTrue(np.allclose(x, r/R, rtol=4*EPS))
        self.assertTrue(np.allclose(Hp, P/(rho*g), rtol=4*EPS, equal_nan=True))
        self.assertTrue(np.allclose(cs2, G1*P/rho, rtol=4*EPS))

        m = fgong.load_fgong('data/modelS.fgong', return_object=True)
        self.assertTrue(np.allclose(m.q, m.m/m.M, rtol=4*EPS))
        self.assertTrue(np.allclose(m.x, m.r/m.R, rtol=4*EPS))
        self.assertTrue(np.allclose(m.Hp, m.P/(m.rho*m.g), rtol=4*EPS, equal_nan=True))
        self.assertTrue(np.allclose(m.cs2, m.Gamma_1*m.P/m.rho, rtol=4*EPS))

if __name__ == '__main__':
    unittest.main()



from tomso import adipls, fgong
import numpy as np
import unittest

tmpfile = 'data/tmpfile'
EPS = np.finfo(float).eps

class TestADIPLSFunctions(unittest.TestCase):

    def test_load_mesa_amdl(self):
        D, A, nmod = adipls.load_amdl('data/mesa.amdl', return_nmod=True)
        self.assertEqual(nmod, 1)
        self.assertEqual(len(A), 601)
        self.assertAlmostEqual(D[0], 1.988205400E+33)
        self.assertAlmostEqual(D[1], 6.204550713E+10)

        a = adipls.load_amdl('data/mesa.amdl', return_object=True)
        self.assertEqual(a.nmod, 1)
        self.assertEqual(len(a.A), 601)
        self.assertAlmostEqual(a.D[0], 1.988205400E+33)
        self.assertAlmostEqual(a.D[1], 6.204550713E+10)

    def test_load_mesa_amdl_fail(self):
        np.savetxt(tmpfile, np.random.rand(100,100))

        with self.assertRaises(IOError):
            D, A = adipls.load_amdl(tmpfile)

        with self.assertRaises(IOError):
            a = adipls.load_amdl(tmpfile, return_object=True)

    def test_amdl_get(self):
        D, A = adipls.load_amdl('data/mesa.amdl')
        M, R, x, G1 = adipls.amdl_get(['M','R','x','G1'], D, A)
        self.assertEqual(M, D[0])
        self.assertEqual(R, D[1])
        self.assertTrue(np.all(x==A[:,0]))
        self.assertTrue(np.all(G1==A[:,3]))

        cs, = adipls.amdl_get(['cs'], D, A)
        cs2 = adipls.amdl_get('cs2', D, A)
        self.assertTrue(np.allclose(cs**2, cs2))

        m = adipls.load_amdl('data/mesa.amdl', return_object=True)
        self.assertEqual(m.M, m.D[0])
        self.assertEqual(m.R, m.D[1])
        self.assertTrue(np.all(m.x==m.A[:,0]))
        self.assertTrue(np.all(m.Gamma_1==m.A[:,3]))
        self.assertTrue(np.all(m.G1==m.A[:,3]))
        self.assertTrue(np.all(m.G1==m.Gamma_1))
        self.assertTrue(np.allclose(m.cs**2, m.cs2))

    def test_amdl_get_cross_check(self):
        D, A = adipls.load_amdl('data/mesa.amdl')
        M, R, P_c, rho_c, r, x, m, q, g, rho, P, Hp, G1, cs2, cs, tau \
            = adipls.amdl_get(['M','R', 'P_c', 'rho_c', 'r', 'x', 'm',
                                'q', 'g', 'rho', 'P', 'Hp', 'G1', 'cs2',
                                'cs', 'tau'], D, A)
        self.assertTrue(np.allclose(q, m/M, rtol=4*EPS))
        self.assertTrue(np.allclose(x, r/R, rtol=4*EPS))
        self.assertTrue(np.allclose(Hp, P/(rho*g), rtol=4*EPS, equal_nan=True))
        self.assertTrue(np.allclose(cs2, G1*P/rho, rtol=4*EPS))

        m = adipls.load_amdl('data/mesa.amdl', return_object=True)
        self.assertTrue(np.allclose(m.q, m.m/m.M, rtol=4*EPS))
        self.assertTrue(np.allclose(m.x, m.r/m.R, rtol=4*EPS))
        self.assertTrue(np.allclose(m.Hp, m.P/(m.rho*m.g), rtol=4*EPS, equal_nan=True))
        self.assertTrue(np.allclose(m.cs2, m.Gamma_1*m.P/m.rho, rtol=4*EPS))

    def test_load_modelS_agsm(self):
        css = adipls.load_agsm('data/modelS.agsm')
        for cs in css:
            self.assertAlmostEqual(cs['M'], 1.989e33)
            self.assertAlmostEqual(cs['R'], 69599062580.0)

        agsm = adipls.load_agsm('data/modelS.agsm', return_object=True)
        for cs in agsm.css:
            self.assertAlmostEqual(cs['M'], 1.989e33)
            self.assertAlmostEqual(cs['R'], 69599062580.0)

        self.assertAlmostEqual(agsm.M, 1.989e33)
        self.assertAlmostEqual(agsm.R, 69599062580.0)
        self.assertTrue(np.allclose(agsm.nu_V, 1/agsm.Pi_V))
        self.assertTrue(np.allclose(agsm.nu_E, 1/agsm.Pi_E))

        for i in range(len(agsm.css)):
            self.assertEqual(i, agsm.index_ln(agsm.l[i], agsm.n[i]))
            self.assertEqual(i, agsm.index_nl(agsm.n[i], agsm.l[i]))

    def test_load_modelS_amde(self):
        css1, eigs1, x1 = adipls.load_amde('data/modelS_nfmode1.amde')
        css2, eigs2, x2 = adipls.load_amde('data/modelS_nfmode2.amde', nfmode=2)
        css3, eigs3, x3 = adipls.load_amde('data/modelS_nfmode3.amde', nfmode=3)

        for cs1, cs2, cs3 in zip(css1, css2, css3):
            self.assertTrue(np.all(cs1==cs2))
            self.assertTrue(np.all(cs1==cs3))
            self.assertAlmostEqual(cs1['M'], 1.989e33)
            self.assertAlmostEqual(cs1['R'], 69599062580.0)

        for eig1, eig2, eig3 in zip(eigs1, eigs2, eigs3):
            self.assertTrue(np.all(eig1[:,1:3] == eig2))
            self.assertTrue(np.all(eig1[:,5:] == eig3))
            self.assertTrue(np.all(eig1[:,0] == x1))
            self.assertTrue(np.all(eig1[:,0] == x2))
            self.assertTrue(np.all(eig1[:,0] == x3))

        amde1 = adipls.load_amde('data/modelS_nfmode1.amde', return_object=True)
        amde2 = adipls.load_amde('data/modelS_nfmode2.amde', return_object=True, nfmode=2)
        amde3 = adipls.load_amde('data/modelS_nfmode3.amde', return_object=True, nfmode=3)

        self.assertTrue(np.all(amde1.eigs[:,:,1:3]==amde2.eigs))
        self.assertTrue(np.all(amde1.x==amde2.x))
        self.assertTrue(np.all(amde1.x==amde3.x))
        self.assertTrue(np.all(amde1.css==amde2.css))
        self.assertTrue(np.all(amde1.css==amde3.css))

    def test_load_amde_nfmode_value_error(self):
        self.assertRaises(ValueError, adipls.load_amde, 'data/modelS_nfmode1.amde', nfmode=4)

    def test_load_modelS_amdl(self):
        D, A, nmod = adipls.load_amdl('data/modelS.amdl', return_nmod=True)
        self.assertEqual(nmod, 1)
        self.assertEqual(len(A), 2482)
        self.assertAlmostEqual(D[0], 1.989e33)
        self.assertAlmostEqual(D[1], 69599062580.0)

    def test_load_modelS_rkr(self):
        css, rkrs = adipls.load_rkr('data/modelS.rkr')

        for cs in css:
            self.assertAlmostEqual(cs['M'], 1.989e33)
            self.assertAlmostEqual(cs['R'], 69599062580.0)

        rkr = adipls.load_rkr('data/modelS.rkr', return_object=True)

        for cs in rkr.css:
            self.assertAlmostEqual(cs['M'], 1.989e33)
            self.assertAlmostEqual(cs['R'], 69599062580.0)

        self.assertTrue(np.all(rkr.K[0]==rkr.K_ln(rkr.l[0], rkr.n[0])))
        self.assertTrue(np.all(rkr.K[0]==rkr.K_nl(rkr.n[0], rkr.l[0])))

    def test_save_mesa_amdl(self):
        D1, A1, nmod1 = adipls.load_amdl('data/mesa.amdl', return_nmod=True)
        adipls.save_amdl(tmpfile, D1, A1, nmod=nmod1)
        D2, A2, nmod2 = adipls.load_amdl(tmpfile, return_nmod=True)
        self.assertTrue(np.all(nmod1 == nmod2))
        self.assertTrue(np.all(len(A1) == len(A2)))
        self.assertTrue(np.all(D1 == D2))
        self.assertTrue(np.all(A1 == A2))

        a1 = adipls.load_amdl('data/mesa.amdl', return_object=True)
        a1.to_file(tmpfile)
        a2 = adipls.load_amdl(tmpfile, return_object=True)
        self.assertTrue(np.all(a1.nmod == a2.nmod))
        self.assertTrue(np.all(a1.nn == a2.nn))
        self.assertTrue(np.all(a1.D == a2.D))
        self.assertTrue(np.all(a1.A == a2.A))

    def test_save_modelS_amdl(self):
        D1, A1, nmod1 = adipls.load_amdl('data/modelS.amdl', return_nmod=True)
        adipls.save_amdl(tmpfile, D1, A1, nmod=nmod1)
        D2, A2, nmod2 = adipls.load_amdl(tmpfile, return_nmod=True)
        self.assertTrue(np.all(nmod1 == nmod2))
        self.assertTrue(np.all(len(A1) == len(A2)))
        self.assertTrue(np.all(D1 == D2))
        self.assertTrue(np.all(A1 == A2))

        a1 = adipls.load_amdl('data/modelS.amdl', return_object=True)
        a1.to_file(tmpfile)
        a2 = adipls.load_amdl(tmpfile, return_object=True)
        self.assertTrue(np.all(a1.nmod == a2.nmod))
        self.assertTrue(np.all(a1.nn == a2.nn))
        self.assertTrue(np.all(a1.D == a1.D))
        self.assertTrue(np.all(a1.A == a1.A))

    def test_fgong_to_amdl_modelS(self):
        D1, A1 = adipls.load_amdl('data/modelS.amdl')
        glob, var = fgong.load_fgong('data/modelS.fgong')
        D2, A2 = adipls.fgong_to_amdl(glob, var, G=6.67232e-8)

        for (x,y) in zip(D1, D2):
            self.assertAlmostEqual(x,y)

        for i in range(len(A1)):
            for (x,y) in zip(A1[i], A2[i]):
                self.assertAlmostEqual(x,y)

        a1 = adipls.load_amdl('data/modelS.amdl', return_object=True,
                              G=6.67232e-8)
        m2 = fgong.load_fgong('data/modelS.fgong', return_object=True,
                              G=6.67232e-8)
        a2 = m2.to_amdl()

        for (x,y) in zip(a1.D, a2.D):
            self.assertAlmostEqual(x,y)

        for i in range(a1.nn):
            for (x,y) in zip(a1.A[i], a2.A[i]):
                self.assertAlmostEqual(x,y)

    def test_fgong_to_amdl_mesa(self):
        D1, A1 = adipls.load_amdl('data/mesa.amdl')
        glob, var = fgong.load_fgong('data/mesa.fgong')
        D2, A2 = adipls.fgong_to_amdl(glob, var, G=6.67428e-8)

        for (x,y) in zip(D1, D2):
            self.assertAlmostEqual(x,y)

        for i in range(len(A1)):
            for (x,y) in zip(A1[i], A2[i]):
                self.assertAlmostEqual(x,y)

        a1 = adipls.load_amdl('data/mesa.amdl', return_object=True,
                              G=6.67428e-8)
        f = fgong.load_fgong('data/mesa.fgong', return_object=True,
                             G=6.67428e-8)
        a2 = f.to_amdl()

        for (x,y) in zip(a1.D, a2.D):
            self.assertAlmostEqual(x,y)

        for i in range(len(a1.A)):
            for (x,y) in zip(a1.A[i], a2.A[i]):
                self.assertAlmostEqual(x,y)

    def test_amdl_to_fgong_modelS(self):
        D, A = adipls.load_amdl('data/modelS.amdl')
        glob1, var1 = fgong.load_fgong('data/modelS.fgong')
        glob2, var2 = adipls.amdl_to_fgong(D, A, G=6.67232e-8)

        for i in [0, 1]:
            self.assertAlmostEqual(glob1[i], glob2[i])

        for i in range(len(var1)-1):
            self.assertAlmostEqual(np.exp(var2[i,1]-var1[i,1]), 1.)
            for j in [0,3,4,9,14]:
                self.assertAlmostEqual(var2[i,j]/var1[i,j], 1.)

        m1 = fgong.load_fgong('data/modelS.fgong', return_object=True,
                              G=6.67232e-8)
        a2 = adipls.load_amdl('data/modelS.amdl', return_object=True,
                              G=6.67232e-8)
        m2 = a2.to_fgong()

        for i in [0, 1]:
            self.assertAlmostEqual(m1.glob[i], m2.glob[i])

        for i in range(len(var1)-1):
            self.assertAlmostEqual(np.exp(m2.var[i,1]-m1.var[i,1]), 1.)
            for j in [0,3,4,9,14]:
                self.assertAlmostEqual(m2.var[i,j]/m1.var[i,j], 1.)

    def test_amdl_to_fgong_mesa(self):
        D, A = adipls.load_amdl('data/mesa.amdl')
        glob1, var1 = fgong.load_fgong('data/mesa.fgong')
        glob2, var2 = adipls.amdl_to_fgong(D, A, G=6.67428e-8)

        for i in [0, 1]:
            self.assertAlmostEqual(glob1[i], glob2[i])

        for i in range(len(var1)-1):
            self.assertAlmostEqual(np.exp(var2[i,1]-var1[i,1]), 1.)
            for j in [0,3,4,9,14]:
                self.assertAlmostEqual(var2[i,j]/var1[i,j], 1.)

        m1 = fgong.load_fgong('data/mesa.fgong', return_object=True,
                              G=6.67428-8)
        a2 = adipls.load_amdl('data/mesa.amdl', return_object=True,
                              G=6.67428e-8)
        m2 = a2.to_fgong()

        for i in [0, 1]:
            self.assertAlmostEqual(m1.glob[i], m2.glob[i])

        for i in range(len(var1)-1):
            self.assertAlmostEqual(np.exp(m2.var[i,1]-m1.var[i,1]), 1.)
            for j in [0,3,4,9,14]:
                self.assertAlmostEqual(m2.var[i,j]/m1.var[i,j], 1.)

    def test_cross_check_css(self):
        css_agsm = adipls.load_agsm('data/modelS.agsm')
        css_amde = adipls.load_amde('data/modelS_nfmode1.amde')[0]
        css_rkr = adipls.load_rkr('data/modelS.rkr')[0]
        self.assertTrue(np.all(css_agsm == css_amde))
        self.assertTrue(np.all(css_agsm == css_rkr))

        agsm = adipls.load_agsm('data/modelS.agsm', return_object=True)
        # amde = adipls.load_amde('data/modelS_nfmode1.amde', return_object=True)
        rkr = adipls.load_rkr('data/modelS.rkr', return_object=True)
        # self.assertTrue(np.all(css_agsm == css_amde))
        self.assertTrue(np.all(agsm.css == rkr.css))

if __name__ == '__main__':
    unittest.main()



"""Functions and constants common to multiple modules."""
import numpy as np
import gzip

DEFAULT_G = 6.67428e-8
"""Default value of the gravitational constant that is shared
by all modules."""

def integrate(y, x):
    """Integral of `y` over `x`, computed using the trapezoidal rule. 
    i.e. :math:`\int _{x[0]} ^x y(x') dx'`."""
    dz = (y[1:]+y[:-1])/2.*np.diff(x)
    return np.hstack((0., np.cumsum(dz)))

def complement(y, x):
    """Complement of integral of `y` over `x`, computed using the
    trapezoidal rule.  i.e. :math:`\int _x^{x[-1]}y(x') dx'`."""
    z = integrate(y, x)
    return z[-1] - z


def tomso_open(filename, *args, **kwargs):
    """Wrapper function to open files ending with `.gz` with built-in
    `gzip` module, otherwise use normal open.  Takes the same
    arguments as `open` and `gzip.open` and returns a file object.
    """
    if filename.lower().endswith('.gz'):
        return gzip.open(filename, *args, **kwargs)
    else:
        return open(filename, *args, **kwargs)


def load_mesa_gyre(filename, mesa_or_gyre):
    """Most MESA and GYRE output files both adhere to a similar columned
    ASCII format, so it makes more sense to have one implementation
    for reading them, rather than re-implementing it in each
    submodule.

    """
    with tomso_open(filename, 'rb') as f:
        lines = f.readlines()

    if mesa_or_gyre == 'mesa':
        header = np.genfromtxt(lines[1:3], names=True, dtype=None, encoding='utf-8')
    elif mesa_or_gyre == 'gyre':
        # the GYRE header might be empty
        try:
            header = np.genfromtxt(lines[2:4], names=True, dtype=None, encoding='utf-8')
        except IndexError:
            header = None
    else:
        raise ValueError("mesa_or_gyre must be either 'mesa' or 'gyre', not %s"
                         % mesa_or_gyre)
            
    data = np.genfromtxt(lines[5:], names=True, dtype=None, encoding='utf-8')

    return header, data



from tomso import common
import numpy as np
import unittest

EPS = np.finfo(float).eps

class TestCommonFunctions(unittest.TestCase):

    def test_integrate_constant(self):
        x = np.linspace(0., 4., 101)
        y = np.ones_like(x)

        self.assertTrue(np.all(common.integrate(y, x) == x))

    def test_integrate_line(self):
        x = np.linspace(0., 4., 101)
        y = x

        self.assertTrue(np.allclose(common.integrate(y, x),
                                    x**2/2., atol=4.*EPS))

    def test_complement_constant(self):
        x = np.linspace(0., 4., 101)
        y = np.ones_like(x)

        self.assertTrue(np.all(common.complement(y, x) == x[-1]-x))

    def test_complement_line(self):
        x = np.linspace(0., 4., 101)
        y = x

        self.assertTrue(np.allclose(common.complement(y, x),
                                    (x[-1]**2-x**2)/2., atol=4.*EPS))

    def test_tomso_open(self):
        with common.tomso_open('data/test.txt', 'rb') as f:
            lines = f.read().decode('utf-8').split('\n')

        with common.tomso_open('data/test.txt.gz', 'rb') as f:
            lines_gz = f.read().decode('utf-8').split('\n')

        for line, line_gz in zip(lines, lines_gz):
            self.assertEqual(line, line_gz)
            

if __name__ == '__main__':
    unittest.main()



from tomso import mesa
import numpy as np
import unittest

tmpfile = 'data/tmpfile'


class TestMESAFunctions(unittest.TestCase):

    def test_load_history(self):
        header, history = mesa.load_history('data/mesa.history')
        self.assertEqual(header['version_number'], 11701)
        self.assertAlmostEqual(header['burn_min1'], 50.0)
        self.assertAlmostEqual(header['burn_min2'], 1000.0)
        self.assertEqual(history['model_number'][-1], 125)
        self.assertEqual(max(history['model_number']), 137)

        h = mesa.load_history('data/mesa.history', return_object=True)
        self.assertEqual(h['version_number'], 11701)
        self.assertAlmostEqual(h['burn_min1'], 50.0)
        self.assertAlmostEqual(h['burn_min2'], 1000.0)
        self.assertEqual(h['model_number'][-1], 125)
        self.assertEqual(max(h['model_number']), 137)
        self.assertTrue(np.allclose(h['log_dt'], np.log10(h['dt'])))

    def test_load_pruned_history(self):
        header, history = mesa.load_history('data/mesa.history', prune=True)
        self.assertEqual(header['version_number'], 11701)
        self.assertAlmostEqual(header['burn_min1'], 50.0)
        self.assertAlmostEqual(header['burn_min2'], 1000.0)
        self.assertEqual(history['model_number'][-1], 125)
        self.assertEqual(max(history['model_number']), 125)

        for i, row in enumerate(history[:-1]):
            self.assertLessEqual(history['model_number'][i],
                                 history['model_number'][i+1])
            self.assertLessEqual(history['star_age'][i],
                                 history['star_age'][i+1])

        h = mesa.load_history('data/mesa.history', prune=True, return_object=True)
        self.assertEqual(h['version_number'], 11701)
        self.assertAlmostEqual(h['burn_min1'], 50.0)
        self.assertAlmostEqual(h['burn_min2'], 1000.0)
        self.assertEqual(h['model_number'][-1], 125)
        self.assertEqual(max(h['model_number']), 125)

        for i, row in enumerate(h[:-1]):
            self.assertLessEqual(h['model_number'][i],
                                 h['model_number'][i+1])
            self.assertLessEqual(h['star_age'][i],
                                 h['star_age'][i+1])

    def test_gzipped_load_history(self):
        header, history = mesa.load_history('data/mesa.history.gz')
        self.assertEqual(header['version_number'], 11701)
        self.assertAlmostEqual(header['burn_min1'], 50.0)
        self.assertAlmostEqual(header['burn_min2'], 1000.0)
        self.assertEqual(history['model_number'][-1], 125)
        self.assertEqual(max(history['model_number']), 137)

        h = mesa.load_history('data/mesa.history.gz', return_object=True)
        self.assertEqual(h['version_number'], 11701)
        self.assertAlmostEqual(h['burn_min1'], 50.0)
        self.assertAlmostEqual(h['burn_min2'], 1000.0)
        self.assertEqual(h['model_number'][-1], 125)
        self.assertEqual(max(h['model_number']), 137)

    def test_load_profile(self):
        header, profile = mesa.load_profile('data/mesa.profile')
        self.assertEqual(header['model_number'], 95)
        self.assertEqual(header['num_zones'], 559)
        self.assertAlmostEqual(header['initial_mass'], 0.9995)
        self.assertAlmostEqual(header['initial_z'], 0.02)

        for i in range(len(profile)):
            self.assertEqual(profile['zone'][i], i+1)

        p = mesa.load_profile('data/mesa.profile', return_object=True)
        self.assertEqual(p['model_number'], 95)
        self.assertEqual(p['num_zones'], 559)
        self.assertAlmostEqual(p['initial_mass'], 0.9995)
        self.assertAlmostEqual(p['initial_z'], 0.02)

        for i in range(len(profile)):
            self.assertEqual(p['zone'][i], i+1)

    def test_load_gzipped_profile(self):
        header, profile = mesa.load_profile('data/mesa.profile.gz')
        self.assertEqual(header['model_number'], 95)
        self.assertEqual(header['num_zones'], 559)
        self.assertAlmostEqual(header['initial_mass'], 0.9995)
        self.assertAlmostEqual(header['initial_z'], 0.02)

        for i in range(len(profile)):
            self.assertEqual(profile['zone'][i], i+1)

        p = mesa.load_profile('data/mesa.profile.gz', return_object=True)
        self.assertEqual(p['model_number'], 95)
        self.assertEqual(p['num_zones'], 559)
        self.assertAlmostEqual(p['initial_mass'], 0.9995)
        self.assertAlmostEqual(p['initial_z'], 0.02)

        for i in range(len(profile)):
            self.assertEqual(p['zone'][i], i+1)

    def test_load_sample(self):
        sample = mesa.load_sample('data/mesa.sample')
        self.assertAlmostEqual(sample['mass/Msun'], 0.9995)
        self.assertAlmostEqual(sample['csound_rms'], 0.0)
        self.assertAlmostEqual(sample['Teff_sigma'], 65.0)
        self.assertAlmostEqual(sample['logL_sigma'], 0.05)
        self.assertAlmostEqual(sample['logg_sigma'], 0.06)
        self.assertAlmostEqual(sample['FeH_sigma'], 0.05)
        for ell in range(4):
            table = sample['l%i' % ell]  # for brevity
            self.assertTrue(np.allclose(table['sigma'], 0.3))
            self.assertTrue(np.allclose(table['chi2term'],
                                        (table['corr']-table['obs'])**2/table['sigma']**2))

            # for sigma in table['err']:
            #     self.assertAlmostEqual(sigma, 0.3)

    def test_load_gzipped_sample(self):
        sample = mesa.load_sample('data/mesa.sample.gz')
        self.assertAlmostEqual(sample['mass/Msun'], 0.9995)
        self.assertAlmostEqual(sample['csound_rms'], 0.0)
        self.assertAlmostEqual(sample['Teff_sigma'], 65.0)
        self.assertAlmostEqual(sample['logL_sigma'], 0.05)
        self.assertAlmostEqual(sample['logg_sigma'], 0.06)
        self.assertAlmostEqual(sample['FeH_sigma'], 0.05)
        for ell in range(4):
            table = sample['l%i' % ell]  # for brevity
            self.assertTrue(np.allclose(table['sigma'], 0.3))
            self.assertTrue(np.allclose(table['chi2term'],
                                        (table['corr']-table['obs'])**2/table['sigma']**2))
            # for sigma in sample['l%i' % ell]['err']:
            #     self.assertAlmostEqual(sigma, 0.3)

    def test_load_astero_results(self):
        results = mesa.load_astero_results('data/simplex_results.data')
        self.assertEqual(len(results), 2)
        self.assertEqual(results[0]['sample'], 1)
        self.assertEqual(results[1]['sample'], 2)

    def test_load_gzipped_astero_results(self):
        results = mesa.load_astero_results('data/simplex_results.data.gz')
        self.assertEqual(len(results), 2)
        self.assertEqual(results[0]['sample'], 1)
        self.assertEqual(results[1]['sample'], 2)

    def test_string_where(self):
        I = mesa.string_where(['a','b','c'], 'b')
        self.assertEqual(I, [1])

        I = mesa.string_where(['b','b','b'], 'b')
        self.assertEqual(I, [0, 1, 2])

        I = mesa.string_where([], 'b')
        self.assertEqual(I, [])

    def test_replace_value(self):
        line = mesa.replace_value('a = foo\n', True)
        self.assertEqual(line, 'a = .true.\n')

        line = mesa.replace_value('a = foo\n', 1)
        self.assertEqual(line, 'a = 1\n')

        line = mesa.replace_value('a = foo\n', 'bar')
        self.assertEqual(line, 'a = bar\n')

    def test_update_inlist(self):
        with open(tmpfile, 'wb') as f:
            f.writelines([b'a = foo\n', b'b = .true.\n', b'c = 1\n'])

        d = {'a': 'bar', 'b': False, 'c': 2}
        mesa.update_inlist(tmpfile, d)

        with open(tmpfile, 'rb') as f:
            lines = f.readlines()

        self.assertEqual(lines, [b'a = bar\n', b'b = .false.\n', b'c = 2\n'])

    def test_update_inlist_no_key(self):
        with open(tmpfile, 'wb') as f:
            f.writelines([b'a = foo\n', b'b = .true.\n', b'c = 1\n'])

        d = {'a': 'bar', 'd': -1}
        with self.assertRaises(IndexError):
            mesa.update_inlist(tmpfile, d)

if __name__ == '__main__':
    unittest.main()



"""
Functions for loading output from the `Cambridge STARS`_ code.

.. _Cambridge STARS: http://www.ast.cam.ac.uk/~stars/index.htm
"""

import numpy as np


def load_out(filename):
    """Reads a STARS `out` file and returns (part of) the summaries and
    profiles in two structured arrays.

    Parameters
    ----------
    filename: str
        Filename of the STARS output file to load.

    Returns
    -------
    summaries: 2-d structured array
        Summaries of each model in the run, similar to MESA's
        `history` files.

    profiles: 3-d structured array
        Model profiles produced at regular intervals during the run.
        The first index of the array is the profile number.
    """
    with open(filename, 'r') as f:
        line = f.readline()

        K = int(line.split()[0])  # number of points in profiles

        summaries = []
        profiles = []

        while line:
            line = f.readline()

            if len(line.split()) < 2:
                continue
            elif line.split()[0] == 'K':
                # found a profile
                dtypes = [('k','int')] + \
                         [(name, 'float') for name in line.split()[1:]]
                data = [f.readline() for i in range(K+(K-1)//10)]
                data = [tuple(map(float, row.split()))
                        for row in data if row != '\n']
                profiles.append(np.array(data, dtype=dtypes))
            elif line.split()[1] == 'dt/age/MH/MHe':
                # found a summary
                # have to do some basic manipulation to make parsing stable
                summary = [int(line.split()[0])]
                for i in range(2):
                    line = f.readline()
                    summary.extend(line.replace('-',' -').replace('E -','E-').split()[1:-1])

                summaries.append(np.array(tuple(summary),
                                          dtype=summary_dtypes[:len(summary)]))

    return np.squeeze(np.vstack(summaries)), np.vstack((profiles))


def load_plot(filename):
    """Reads a STARS `plot` file into a structured array.

    Parameters
    ----------
    filename: str
        Filename of the STARS plot file to load.

    Returns
    -------
    plot: 2-d structured array
        Data for evolutionary track, with one row per model along the
        track. e.g. age, effective temperature, etc.

    """
    return np.loadtxt(filename, dtype=plot_dtypes)


summary_dtypes = [('n','int'), ('dt', 'float'),
                  ('tn', 'float'), ('P', 'float'),
                  ('LH', 'float'), ('Lth', 'float'), ('H1_cntr','float'),
                  ('He4_cntr', 'float'), ('C12_cntr', 'float'),
                  ('N14_cntr', 'float'), ('O16_cntr','float'),
                  ('Ne20_cntr', 'float'), ('He3_cntr','float'),
                  ('psi_cntr', 'float'), ('dens_cntr','float'),
                  ('temp_cntr', 'float'), ('age','float'),
                  ('tKH', 'float'),('rlf', 'float'),
                  ('LHe','float'),('Lnu', 'float'), ('H1_srfc', 'float'),
                  ('He4_srfc', 'float'), ('C12_srfc', 'float'),
                  ('N14_srfc', 'float'), ('O16_srfc', 'float'),
                  ('Ne20_srfc', 'float'), ('He3_srfc', 'float'),
                  ('psi_srfc', 'float'), ('dens_srfc', 'float'),
                  ('temp_srfc', 'float'), ('MH', 'float'), ('MHe','float'),
                  ('Mb', 'float'), ('dM', 'float'), ('LC','float'),
                  ('m', 'float'), ('H1_Tmax', 'float'),
                  ('He4_Tmax', 'float'), ('C12_Tmax', 'float'),
                  ('N14_Tmax', 'float'), ('O16_Tmax', 'float'),
                  ('Ne20_Tmax', 'float'), ('He3_Tmax', 'float'),
                  ('psi_Tmax', 'float'), ('dens_Tmax', 'float'),
                  ('temp_Tmax', 'float')]


plot_dtypes = [('n','int'), ('age', 'float'), ('logR', 'float'),
               ('logTeff', 'float'), ('logL', 'float'), ('M', 'float'),
               ('M_H','float'), ('M_He', 'float'), ('L_H', 'float'),
               ('L_He', 'float'), ('L_C', 'float'), ('mconv1', 'float'),
               ('m_conv2','float'), ('m_conv3','float'), ('m_conv4','float'),
               ('m_conv5','float'), ('m_conv6','float'), ('m_conv7','float'),
               ('m_conv8','float'), ('m_conv9','float'), ('m_conv10','float'),
               ('m_conv11','float'), ('m_conv12','float'),
               ('m_eps_H_max', 'float'), ('m_eps_He_max', 'float'),
               ('logkappa', 'float'), ('dt', 'float'),
               ('X_Hs', 'float'), ('X_Hes', 'float'), ('H_Cs', 'float'),
               ('X_Ns','float'), ('X_Os', 'float'), ('X_He3s', 'float'),
               ('R/R_RL', 'float'), ('J_1', 'float'), ('P', 'float'),
               ('R_sep', 'float'), ('M1+M2','float'), ('J_orb', 'float'),
               ('J_1+J_2', 'float'), ('J_tot','float'), ('omega_orb', 'float'),
               ('omega_1', 'float'), ('I_1','float'), ('I_orb', 'float'),
               ('Mdot', 'float'), ('m_shell1','float'), ('m_shell2','float'),
               ('m_shell3','float'), ('m_shell4','float'),
               ('m_shell5','float'), ('m_shell6','float'),
               ('m_shell7','float'), ('m_shell8','float'),
               ('m_shell9','float'), ('m_shell10','float'),
               ('m_shell11','float'), ('m_shell12','float'),
               ('m_th1','float'), ('m_th2','float'), ('m_th3','float'),
               ('m_th4','float'), ('m_th5','float'), ('m_th6','float'),
               ('m_th7','float'), ('m_th8','float'), ('m_th9','float'),
               ('m_th10','float'), ('m_th11','float'), ('m_th12','float'),
               ('M_ce', 'float'), ('R_ce', 'float'),
               ('logRho_c', 'float'), ('logT_c', 'float')]



# -*- coding: utf-8 -*-

"""Functions and classes for reading and writing ADIPLS binary output.  Many
return or contain what I call ``cs`` arrays.  These are defined in Section 8.2 of
the `ADIPLS documentation`_.  They are structured arrays containing
various scalar results from the frequency calculation.

    .. _ADIPLS documentation: https://sourceforge.net/p/mesa/code/HEAD/tree/trunk/adipls/adipack.c/notes/adiab.prg.c.pdf
"""
import numpy as np
import warnings
from .common import integrate, DEFAULT_G

def read_one_cs(f):
    """Utility function to parse one ``cs`` array from a binary file
    handle ``f``."""
    cs = np.fromfile(f, dtype=cs_floats, count=1)
    cs = cs.astype(cs_dtypes, copy=False)
    return cs


def load_pointwise_data(filename, ncols):
    """Utility function for common structure of ADIPLS data that has a
    value at each point in a stellar model. e.g. eigenfunction or
    kernel files.

    Parameters
    ----------
    filename: str
        Name of the file to be read.
    ncols: int
        Number of columns in the data.

    Returns
    -------
    css: structured array
        The ``cs`` arrays for each mode.
    data: list of arrays
        The point-wise data arrays for each mode.

    """
    css = []
    data = []

    with open(filename, 'rb') as f:
        while True:
            if not f.read(4): break
            css.append(read_one_cs(f))
            nnw = np.fromfile(f, dtype='i', count=1)[0]
            row = np.fromfile(f, dtype='d', count=ncols*nnw).reshape((-1, ncols))
            data.append(row)
            f.read(4)

    return np.squeeze(css), np.squeeze(data)


def load_agsm(filename, return_object=False):
    """Reads an ADIPLS grand summary file and returns a structured array.

    If `return_object` is `True`, instead returns an
    `ADIPLSGrandSummary` object.  This will become default behaviour
    from v0.0.12.  The old behaviour will be dropped completely from
    v0.1.0.

    Parameters
    ----------
    filename: str
        Name of the grand summary file, usually starting or ending
        with ``agsm``.

    Returns
    -------
    css: structured array
        The ``cs`` arrays for each mode.

    """

    css = []

    with open(filename, 'rb') as f:
        while True:
            if not f.read(4): break
            css.append(read_one_cs(f))
            f.read(4)

    if return_object:
        return ADIPLSGrandSummary(np.squeeze(css))
    else:
        warnings.warn("From tomso 0.1.0+, `adipls.load_agsm` will only "
                      "return an `FGONG` object: use `return_object=True` "
                      "to mimic future behaviour",
                      FutureWarning)
        return np.squeeze(css)


def load_amde(filename, nfmode=1, return_object=False):
    """Reads an ADIPLS eigenfunction file written with the specified value
    of ``nfmode`` in the input file (either 1, 2 or 3).

    Parameters
    ----------
    filename: str
        Name of the eigenfunction file, usually starting or ending
        with ``amde``.
    nfmode: int, optional
        ADIPLS's ``nfmode`` parameter, which determines the format of
        the eigenfunction data.  See Section 8.4 of the `ADIPLS
        documentation`_ for details of the output.  Note that for
        ``nfmode=2`` or ``3``, the fractional radius is returned as an
        extra (third) component.

    Returns
    -------
    css: structured array
        The ``cs`` arrays for each mode.
    eigs: list of arrays
        The eigenfunction arrays for each mode.  Each array has seven
        columns: the fractional radius :math:`x` and six columns of
        ADIPLS's :math:`y` matrix.  The first four columns of
        :math:`y` are defined by equation (2.5) of the documentation
        and the last two by equations (4.4) and (4.6).
    x: array
        Fractional radius co-ordinate ``x`` of the eigenfunctions.  If
        ``nfmode=1``, the radial co-ordinate is also stored in
        ``eigs[...,0]`` but it's returned anyway so that the returned
        data always has the same structure.
    """

    if nfmode == 1:
        css, data = load_pointwise_data(filename, 7)
        x = data[0,:,0]
    elif nfmode == 2 or nfmode == 3:
        # thanks to Vincent Boening for this
        ncols = 2
        css = []
        data = []
        with open(filename, 'rb') as f:
            f.read(4)
            nnw = np.fromfile(f, dtype='i', count=1)[0]
            x = np.fromfile(f, dtype='d', count=nnw)
            f.read(4)

            while True:
                if not f.read(4): break
                css.append(read_one_cs(f))
                row = np.fromfile(f, dtype='d', count=ncols*nnw).reshape((-1, ncols))
                data.append(row)
                f.read(4)
    else:
        raise ValueError('nfmode must be 1, 2 or 3 but got %i' % nfmode)

    if return_object:
        return ADIPLSEigenfunctions(np.squeeze(css), np.squeeze(data), x=x, nfmode=nfmode)
    else:
        warnings.warn("From tomso 0.1.0+, `adipls.load_agsm` will only "
                      "return an `FGONG` object: use `return_object=True` "
                      "to mimic future behaviour",
                      FutureWarning)
        return np.squeeze(css), np.squeeze(data), x

def load_amdl(filename, return_nmod=False, live_dangerously=False,
              return_object=False, G=DEFAULT_G):
    """Reads an ADIPLS model file.  See Section 5 of the `ADIPLS
    documentation`_ for details.

    If `return_object` is `True`, instead returns an
    `ADIPLSStellarModel` object.  This will become default behaviour
    from v0.0.12.  The old behaviour will be dropped completely from
    v0.1.0.

    Parameters
    ----------
    filename: str
        Name of the model file, usually starting or ending with ``amdl``.
    return_nmod: bool, optional
        If `True`, return the ``nmod`` parameter in the file.
    live_dangerously: bool, optional
        If `True`, load the file even if it looks like it might be
        too large for an AMDL file (i.e. has more than a million points).

    Returns
    -------
    D: 1-d NumPy array
        Global data, as defined by eq. (5.2) of the ADIPLS
        documentation.
    A: 2-d NumPy array
        Point-wise data, as defined by eq. (5.1) of the ADIPLS
        documentation.
    nmod: int, optional
        The model number.  I'm not sure what it's used for but it
        doesn't seem to matter.  Only returned if `return_nmod=True`.

    """

    with open(filename, 'rb') as f:
        f.read(4)
        nmod = np.fromfile(f, dtype='i', count=1)[0]
        nn = np.fromfile(f, dtype='i', count=1)[0]
        if not live_dangerously and nn > 1000000:
            raise IOError("Model appears to have %i points; "
                          "it probably isn't an AMDL file. "
                          "If you're sure that it is, try again "
                          "with live_dangerously=True" % nn)

        D = np.fromfile(f, dtype='d', count=8)
        A = np.fromfile(f, dtype='d', count=6*nn).reshape((-1,6))
        f.read(4)
        # check that this is the end of the file

    if return_object:
        return ADIPLSStellarModel(D, A, nmod=nmod, G=G)
    else:
        warnings.warn("From tomso 0.1.0+, `adipls.load_amdl` will only "
                      "return an `ADIPLSStellarModel` object: use "
                      "`return_object=True` to mimic future behaviour",
                      FutureWarning)
        if return_nmod:
            return D, A, int(nmod)
        else:
            return D, A


def load_rkr(filename, return_object=False):
    """Reads an ADIPLS rotational kernel file.

    Parameters
    ----------
    filename: str
        Name of the kernel file, usually starting or ending with
        ``rkr``.

    Returns
    -------
    css: structured array
        The ``cs`` arrays for each mode.
    rkrs: list of arrays
        The kernel arrays for each mode.  Each array has two columns:
        the fractional radius :math:`x` and the kernel :math:`K(x)`.

    """

    if return_object:
        return ADIPLSRotationKernels(*load_pointwise_data(filename, 2))
    else:
        warnings.warn("From tomso 0.1.0+, `adipls.load_rkr` will only "
                      "return an `ADIPLSRotationKernels` object: use "
                      "`return_object=True` to mimic future behaviour",
                      FutureWarning)
    return load_pointwise_data(filename, 2)


def save_amdl(filename, D, A, nmod=0):
    """Writes an ADIPLS model file, given data in the same form as
    returned by :py:meth:`~tomso.adipls.load_amdl`.  See Section 5 of
    the `ADIPLS documentation`_ for details.

    Parameters
    ----------
    filename: str
        Name of the model file, usually starting or ending with amdl.
    D: 1-d NumPy array
        Global data, as defined by eq. (5.2) of the `ADIPLS
        documentation`_.
    A: 2-d NumPy array
        Point-wise data, as defined by eq. (5.1) of the `ADIPLS
        documentation`_.
    nmod: int, optional
        The model number.  I'm not sure what it's used for but it
        doesn't seem to matter.

    """
    nn = len(A)
    length = np.array(8*(1+8+6*nn), dtype=np.int32)
    with open(filename, 'wb') as f:
        length.tofile(f)
        np.array((nmod,), dtype=np.int32).tofile(f)
        np.array((nn,), dtype=np.int32).tofile(f)
        D.tofile(f)
        A.tofile(f)
        length.tofile(f)


def amdl_get(key_or_keys, D, A, G=DEFAULT_G):
    """Retrieves physical properties of an AMDL model from the ``D`` and
    ``A`` arrays.

    Parameters
    ----------
    keys: list of strs
        A desired variable or a list of desired variables.  Current
        options are:

        - ``M``: total mass (float)
        - ``R``: photospheric radius (float)
        - ``P_c``: central pressure (float)
        - ``rho_c``: central density (float)
        - ``r``: radius (array)
        - ``x``: fractional radius (array)
        - ``m``: mass co-ordinate (array)
        - ``q``: fractional mass co-ordinate (array)
        - ``g``: gravity (array)
        - ``rho``: density (array)
        - ``P``: pressure (array)
        - ``AA``: Ledoux discriminant (array)
        - ``Hp``: pressure scale height (array)
        - ``G1``: first adiabatic index (array)
        - ``cs2``: sound speed squared (array)
        - ``cs``: sound speed (array)
        - ``tau``: acoustic depth

        For example, if ``D`` and ``A`` have been returned from
        :py:meth:`~tomso.adipls.load_amdl`, you could use

        >>> M, m = adipls.amdl_get(['M', 'm'], D, A)

        to get the total mass and mass co-ordinate.  If you only want
        one variable, you don't need to use a list.  The return type
        is just the one corresponding float or array.  So, to get a
        single variable you could use either

        >>> x, = adipls.amdl_get(['x'], D, A)

        or

        >>> x = adipls.amdl_get('x', D, A)

    D: 1-d array
        Global data, as defined by eq. (5.2) of the `ADIPLS
        documentation`_ and returned by
        :py:meth:`~tomso.adipls.load_amdl`.
    A: 2-d array
        Point-wise data, as defined by eq. (5.1) of the `ADIPLS
        documentation`_ and returned by
        :py:meth:`~tomso.adipls.load_amdl`.

    Returns
    -------
    output: list of floats and arrays
        A list returning the floats or arrays in the order requested
        by the parameter ``keys``.

    """
    M, R, P_c, rho_c = D[:4]
    x = A[:,0]                                      # fractional radius
    q = A[:,1]*x**3                                 # fractional mass
    m = q*M                                         # mass
    r = x*R                                         # radius
    G1 = A[:,3]                                     # first adiabatic index
    AA = A[:,4]                                     # Ledoux disciminant

    # we can safely ignore 0/0s here
    with np.errstate(invalid='ignore'):
        g = G*m/r**2
        rho = A[:,5]*m/r**3/4./np.pi                # density
        rho[x==0] = rho_c
        P = G*m*rho/G1/r/A[:,2]                     # pressure
        P[x==0] = P_c

    Hp = P/(rho*g)                                  # pressure scale height
    cs2 = G1*P/rho                                  # sound speed squared
    cs = np.sqrt(cs2)                               # sound speed
    tau = -integrate(1./cs[::-1], r[::-1])[::-1]    # acoustic depth

    if type(key_or_keys) == str:
        keys = [key_or_keys]
        just_one = True
    else:
        keys = key_or_keys
        just_one = False

    output = []
    for key in keys:
        if key == 'M': output.append(M)
        elif key == 'R': output.append(R)
        elif key == 'P_c': output.append(P_c)
        elif key == 'rho_c': output.append(rho_c)
        elif key == 'r': output.append(r)
        elif key == 'x': output.append(x)
        elif key == 'm': output.append(m)
        elif key == 'q': output.append(q)
        elif key == 'g': output.append(g)
        elif key == 'rho': output.append(rho)
        elif key == 'P': output.append(P)
        elif key == 'AA': output.append(AA)
        elif key == 'Hp': output.append(Hp)
        elif key == 'G1': output.append(G1)
        elif key == 'cs2': output.append(cs2)
        elif key == 'cs': output.append(cs)
        elif key == 'tau': output.append(tau)
        else: raise ValueError('%s is not a valid key for adipls.amdl_get' % key)

    if just_one:
        assert(len(output) == 1)
        return output[0]
    else:
        return output


def kernels(cs, eig, D, A, G=DEFAULT_G, alpha=None):
    """Returns the density and squared sound speed kernels.  I have tried
    to make this as notationally similar to Gough & Thompson (1991) as
    possible.  The kernels are normalized to have unit integrals over
    the radius *r*.

    Parameters
    ----------
    cs: structured array
        The ``cs`` array for the mode.
    eig: np.array, shape(N,7)
        Eigenfunction data for the mode, as returned by
        :py:meth:`~tomso.adipls.load_amde`.
    D: 1-d array
        Global data, as defined by eq. (5.2) of the `ADIPLS
        documentation`_ and returned by
        :py:meth:`~tomso.adipls.load_amdl`.
    A: 2-d array
        Point-wise data, as defined by eq. (5.1) of the `ADIPLS
        documentation`_ and returned by
        :py:meth:`~tomso.adipls.load_amdl`.
    G: float, optional
        The gravitational constant.
    alpha: float, optional
        Coefficient of the complementary function.  If ``None``, computed
        as in Michael Thompson's kernel code.

    Returns
    -------
    K_cs2: np.array, length N
        The sound speed squared structure kernel.
    K_rho: np.array, length N
        The density structure kernel.

    """

    def integrate(yy, xx):
        """\int _x[0] ^x y(x) dx"""
        dz = (yy[1:]+yy[:-1])/2.*np.diff(xx)
        return np.hstack((0., np.cumsum(dz)))

    def complement(yy, xx):
        """\int _x ^x[-1] y(x) dx"""
        zz = integrate(yy, xx)
        return zz[-1] - zz

    ell = cs['ell']
    M, R, P_c, rho_c = D[:4]                # mass and radius from FGONG
    y = eig.T
    sigma2 = cs['sigma2']
    omega = np.sqrt(sigma2*G*M/R**3)        # convert to angular frequency
    L2 = ell*(ell+1)

    x = A[:,0]
    r = x*R                                 # radial co-ordinate
    m = A[:,1]*x**3*M                       # mass co-ordinate
    g = G*m/r**2                            # gravity
    g[x==0] = 0.
    rho = A[:,1]*A[:,5]*M/4./np.pi/R**3     # density
    rho[x==0] = rho_c
    G1 = A[:,3]                             # first adiabatic index
    P = G*m*rho/G1/r/A[:,2]                 # pressure
    P[x==0] = P_c
    cs2 = G1*P/rho                          # square of the sound speed
    A1 = A[:,1]
    A2 = A[:,2]
    Vg = A2[:]
    drho_dr = -(A[:,4]+A[:,2])*rho/r        # density gradient
    drho_dr[x==0] = 0.

    xi_r = y[1]*R

    if ell == 0:
        xi_h = 0.*xi_r  # radial modes have zero horizontal component
        chi = Vg/x*(y[1]-sigma2/A1/x*y[2])
        dxi_r_dr = chi - 2.*y[1]/x
        dPhi_dr = -4.*np.pi*G*rho*xi_r
        Phi = -complement(dPhi_dr, r)  # but actually you don't even need it
    elif ell > 0:
        xi_h = y[2]*R/L2
        eta = L2*A1/sigma2
        chi = Vg/x*(y[1]-y[2]/eta-y[3])
        dxi_r_dr = chi - 2.*y[1]/x + y[2]/x
        dPhi_dr = -g/x*(y[3] + y[4]) - y[3]*R*(4.*np.pi*G*rho - 2.*g/r)
        Phi = -g*R*y[3]
    else:
        raise ValueError('ell must be non-negative')

    chi[x==0] = 0.
    dxi_r_dr[x==0] = 0.
    dPhi_dr[x==0] = 0.
    Phi_r = Phi/r
    Phi_r[x==0] = 0.

    S = np.trapz((xi_r**2 + L2*xi_h**2)*rho*r**2, r)

    K_cs2 = rho*cs2*chi**2*r**2  # c.f. equation (60)
    K_cs2 = K_cs2/S/omega**2/2.

    # following InversionKit (103)
    K_rho = cs2*chi**2 - omega**2*(xi_r**2+L2*xi_h**2) \
        - 2.*g*xi_r*(chi - dxi_r_dr) \
        + 4.*np.pi*G*rho*xi_r**2 \
        - 4.*np.pi*G*complement((2.*rho*chi+xi_r*drho_dr)*xi_r, r) \
        + 2.*(xi_r*dPhi_dr + L2*xi_h*Phi_r)
    K_rho = K_rho*rho*r**2/2./S/omega**2

    comp = rho*r**2
    if alpha is None:
        alpha = np.trapz(K_rho*comp, r)/np.trapz(comp*comp, r)

    K_rho = K_rho - alpha*comp

    return K_cs2, K_rho


def fgong_to_amdl(glob, var, G=DEFAULT_G):
    """Converts FGONG data (in the form of `glob` and `var`, as returned
    by :py:meth:`~tomso.fgong.load_fgong`) into ADIPLS binary data,
    which can be saved using :py:meth:`~tomso.adipls.save_amdl`.

    The output should be identical (to within a few times machine
    error) to the output of ``fgong-amdl.d`` tool distributed with
    ADIPLS.

    Parameters
    ----------
    glob: NumPy array
        The scalar (or global) variables for the stellar model
    var: NumPy array
        The point-wise variables for the stellar model. i.e. things
        that vary through the star like temperature, density, etc.
    G: float, optional
        Newton's gravitational constant in cgs units.

    Returns
    -------
    D: 1-d array
        Global data, as defined by eq. (5.2) of the `ADIPLS
        documentation`_.
    A: 2-d array
        Point-wise data, as defined by eq. (5.1) of the `ADIPLS
        documentation`_.

    """
    warnings.warn("From tomso 0.1.0+, FGONG should be converted to "
                  "AMDL by loading an `FGONG` object and using its "
                  "`to_amdl` function.  To mimic the future behaviour, "
                  "load an FGONG object by using `fgong.load_fgong` "
                  "with `return_object=True`.",
                  FutureWarning)

    M, R = glob[:2]
    r, P, rho, G1, AA = var[::-1,[0,3,4,9,14]].T
    m = np.exp(var[::-1,1])*M

    ioff = (0 if r[0] < 1e6 else 1)
    nn = len(var) + ioff

    # convert profile
    A = np.zeros((nn, 6))

    # we can safely ignore division by 0 here
    with np.errstate(divide='ignore', invalid='ignore'):
        A[ioff:,0] = r/R
        A[ioff:,1] = m/M/(r/R)**3
        A[ioff:,2] = G*m*rho/(G1*P*r)
        A[ioff:,3] = G1
        A[ioff:,4] = AA
        A[ioff:,5] = 4.*np.pi*rho*r**3/m

    A[0,0] = 0.
    A[0,1] = 4.*np.pi/3.*rho[0]*R**3/M
    A[0,2] = 0.
    A[0,3] = G1[0]
    A[0,4] = 0.
    A[0,5] = 3.

    # convert header
    D = np.zeros(8)
    D[0] = M
    D[1] = R
    D[2] = P[0]
    D[3] = rho[0]

    # second derivatives at centre are given
    if glob[10] < 0.:
        D[4] = -glob[10]/G1[0]
        D[5] = -glob[11]
    else:
        D[4] = 4.*np.pi/3.*G*(rho[0]*R)**2/(P[0]*G1[0])
        # D[5] = np.nanmax((A[1:,4]/A[1:,0]**2)[A[1:,0]<0.05])
        # D[5] = np.max((D[5], 0.))+D[4]
        D[5] = D[4]

    D[6] = -1.
    D[7] = 0.

    if A[-1,4] <= 10.:
        # chop off outermost point
        A = A[:-1]
        nn -= 1

    return D, A


def amdl_to_fgong(D, A, G=DEFAULT_G):
    """Converts ADIPLS binary data (in the form of `D` and `A`, as
    returned by :py:meth:`~tomso.adipls.load_amdl`) into FGONG data,
    which can be saved using :py:meth:`~tomso.fgong.save_fgong`.

    Designed to be the inverse of the ``fgong-amdl.d`` tool
    distributed with ADIPLS, modulo the fact that various thermal
    variables (e.g. temperature and luminosity) are not stored in the
    AMDL format, and that the innermost point may have been truncated.
    It's impossible to tell when reversing the process, so we assume
    not.

    The output should be identical (to within a few times machine
    error) of going from FGONG to AMDL and back again.

    Parameters
    ----------
    D: 1-d array
        Global data, as defined by eq. (5.2) of the `ADIPLS
        documentation`_.
    A: 2-d array
        Point-wise data, as defined by eq. (5.1) of the `ADIPLS
        documentation`_.
    G: float, optional
        Newton's gravitational constant in cgs units.

    Returns
    -------
    glob: NumPy array
        The scalar (or global) variables for the stellar model
    var: NumPy array
        The point-wise variables for the stellar model. i.e. things
        that vary through the star like temperature, density, etc.

    """
    warnings.warn("From tomso 0.1.0+, AMDL should be converted to "
                  "FGONG by loading an `ADIPLSStellarModel` object "
                  "and using its `to_fgong` function.  To mimic the "
                  "future behaviour, load an FGONG object by using "
                  "`adipls.load_amdl` with `return_object=True`.",
                  FutureWarning)
    M, R = D[:2]

    glob = np.zeros(15)
    var = np.zeros((len(A), 40))

    r = A[:,0]*R
    q = A[:,1]*A[:,0]**3
    m = q*M
    G1 = A[:,3]
    AA = A[:,4]

    # we can safely ignore division by 0 here
    with np.errstate(divide='ignore', invalid='ignore'):
        lnq = np.log(q)
        rho = A[:,5]*m/(4.*np.pi*r**3)
        P = G*m*rho/(G1*r*A[:,2])

    P[0] = D[2]
    rho[0] = D[3]

    var[::-1,0] = r
    var[::-1,1] = lnq
    var[::-1,3] = P
    var[::-1,4] = rho
    var[::-1,9] = G1
    var[::-1,14] = AA

    glob[0] = M
    glob[1] = R
    glob[10] = -D[4]*G1[0]
    glob[11] = -D[5]

    return glob, var


cs_dtypes = [('xmod','float'), ('M','float'), ('R','float'),
             ('P_c','float'), ('rho_c','float'), ('D_5','float'),
             ('D_6','float'), ('D_7','float'), ('D_8','float'),
             ('A_2(x_s)','float'), ('A_5(x_s)','float'),
             ('x_1','float'), ('sigma2_Omega','float'),
             ('x_f','float'), ('fctsbc','int'), ('fcttbc','int'),
             ('lambda','float'), ('ell','int'), ('enn','int'),
             ('sigma2','float'), ('sigma2_c','float'),
             ('y_1,max','float'), ('x_max', 'float'), ('E','float'),
             ('Pi_E','float'), ('Pi_V','float'), ('nu_V','float'),
             ('ddsig','float'), ('ddsol','float'),
             ('y_1(x_s)','float'), ('y_2(x_s)','float'),
             ('y_3(x_s)','float'), ('y_4(x_s)','float'),
             ('z_1,max','float'), ('xhat_max','float'),
             ('beta_nl','float'), ('nu_Ri','float'), ('emm','int')]

for i in range(len(cs_dtypes), 50):
    cs_dtypes.append(('col%i' % i, 'float'))

cs_floats = [(k, 'float') for (k,v) in cs_dtypes]


class ADIPLSStellarModel(object):
    """A class that contains and allows one to manipulate the data in a
    stellar model stored in ADIPLS's internal binary model format.
    See Section 5 of the `ADIPLS documentation`_ for details.

    The main attributes are the **D** and **A** arrays, which follow
    the definitions in the ADIPLS documentation.  The data in these
    arrays can be accessed via the attributes with more
    physically-meaningful names (e.g. the radius is
    ``ADIPLSStellarModel.r``).

    Some of these values can also be set via the attributes if doing
    so is unambiguous. For example, the fractional radius **x** is not a
    member of the **var** array but setting **x** will assign the actual
    radius **r**, which is the first column of **var**.  Values that are
    settable are indicated in the list of parameters.

    Parameters
    ----------
    D: 1-d array
        Global data, as defined by eq. (5.2) of the ADIPLS
        documentation.
    A: 2-d array
        Point-wise data, as defined by eq. (5.1) of the ADIPLS
        documentation.
    nmod: int, optional
        The model number.  I'm not sure what it's used for but it
        doesn't seem to matter.
    G: float, optional
        Value for the gravitational constant.  If not given (which is
        the default behaviour), we use ``glob[14]`` if its close to
        the module-wise default value.  Otherwise, we use the
        module-wise default value.

    Attributes
    ----------
    nn: int
        number of points in stellar model (i.e. number of rows in **A**)
    M: float, settable
        total mass
    R: float, settable
        photospheric radius
    P_c: float, settable
        central pressure
    rho_c: float, settable
        central density
    x: NumPy array, settable
        fractional radius co-ordinate
    q: NumPy array, settable
        fractional mass co-ordinate
    lnq: NumPy array, settable
        natural logarithm of the fractional mass co-ordinate
    Vg: NumPy array
        homology invariant *V/Gamma_1*
    Gamma_1: NumPy array, settable
        first adiabatic index, aliased by **G1**
    G1: NumPy array, settable
        first adiabatic index, alias of **Gamma_1**
    AA: NumPy array, settable
        Ledoux discriminant
    U: NumPy array
        homology invariant *dlnm/dlnr*
    V: NumPy array
        homology invariant *dlnP/dlnr*
    r: NumPy array, settable
        radius co-ordinate
    m: NumPy array, settable
        mass co-ordinate
    P: NumPy array
        pressure
    rho: NumPy array
        density
    g: NumPy array
        local gravitational acceleration
    Hp: NumPy array
        pressure scale height
    Hrho: NumPy array
        density scale height
    N2: NumPy array
        squared Brunt–Väisälä (angular) frequency
    cs2: NumPy array
        squared adiabatic sound speed
    cs: NumPy array
        adiabatic sound speed

    """
    def __init__(self, D, A, nmod=0, G=DEFAULT_G):
        self.D = D
        self.A = A
        self.nmod = nmod
        self.G = G

    def __len__(self):
        return len(self.A)

    def to_file(self, filename):
        """Save the model to an ADIPLS binary stellar model file (usually
        either starting or ending with `amdl`).

        Parameters
        ----------
        filename: str
            Filename to which the data is written.
        """
        save_amdl(filename, self.D, self.A, nmod=self.nmod)

    def to_fgong(self, reverse=True):
        """Convert the model to an ``FGONG`` object.

        Note that the ADIPLS binary format only has the data necessary
        to compute adiabiatic stellar oscillations, so the FGONG will
        be missing some data (e.g. temperature, luminosity).
        """
        from .fgong import FGONG

        # `amdl_to_fgong` already reverses the data by default
        if reverse:
            return FGONG(*amdl_to_fgong(self.D, self.A, G=self.G))
        else:
            return FGONG(*amdl_to_fgong(self.D, self.A[::-1], G=self.G))

    def to_gyre(self, version=None):
        """Convert the model to an ``GYREStellarModel`` object.

        Note that the ADIPLS binary format only has the data necessary
        to compute adiabiatic stellar oscillations, so the GYRE
        stellar model will be missing some data (e.g. temperature,
        luminosity).
        """
        from .gyre import gyre_header_dtypes, gyre_data_dtypes, GYREStellarModel

        if version is None:
            version = max([k for k in gyre_header_dtypes.keys()])

        header = np.zeros(1, gyre_header_dtypes[version])
        header['M'] = self.D[0]
        header['R'] = self.D[1]
        header['L'] = 42.0
        header['version'] = version

        data = np.ones(self.nn, gyre_data_dtypes[version])
        g = GYREStellarModel(header[0], data, G=self.G)

        g.r = self.r
        g.m = self.m
        g.P = self.P
        g.rho = self.rho
        g.Gamma_1 = self.Gamma_1
        g.N2 = self.N2

        # GYRE doesn't know if it's doing adiabatic or non-adiabatic
        # modes when it reads the file, so it does some calculations
        # expecting meaningful data.  We fudge this so we don't get
        # FPEs.
        g.kappa = 42.0
        g.L_r = 42.0
        if version < 101:
            g.data['eps_tot'] = 42.0
        else:
            g.data['eps'] = 42.0

        g.data['k'] = np.arange(self.nn) + 1
        return g


    # AMDL parameters that can be derived from data
    @property
    def nn(self): return len(self.A)

    # Various properties for easier access to the data in `glob` and
    # `var`.

    @property
    def M(self): return self.D[0]

    @M.setter
    def M(self, val): self.D[0] = val

    @property
    def R(self): return self.D[1]

    @R.setter
    def R(self, val): self.D[1] = val

    @property
    def P_c(self): return self.D[2]

    @P_c.setter
    def P_c(self, val): self.D[2] = val

    @property
    def rho_c(self): return self.D[3]

    @rho_c.setter
    def rho_c(self, val): self.D[3] = val

    @property
    def x(self): return self.A[:,0]

    @x.setter
    def x(self, val): self.A[:,0] = val

    @property
    def q(self): return self.A[:,1]*self.x**3

    @q.setter
    def q(self, val): self.A[:,1] = val/self.x**3

    @property
    def Vg(self): return self.A[:,2]

    @Vg.setter
    def Vg(self, val): self.A[:,2] = val

    @property
    def Gamma_1(self): return self.A[:,3]

    @Gamma_1.setter
    def Gamma_1(self, val): self.A[:,3] = val

    @property
    def G1(self): return self.A[:,3]

    @G1.setter
    def G1(self, val): self.A[:,3] = val

    @property
    def AA(self): return self.A[:,4]

    @AA.setter
    def AA(self, val): self.A[:,4] = val

    @property
    def U(self): return self.A[:,5]

    @U.setter
    def U(self, val): self.A[:,5] = val

    @property
    def V(self): return self.Vg*self.Gamma_1

    @V.setter
    def V(self, val): self.Vg = val/self.Gamma_1

    @property
    def r(self): return self.x*self.R

    @r.setter
    def r(self, val): self.x = val/self.R

    @property
    def m(self): return self.q*self.M

    @m.setter
    def m(self, val): self.q = val/self.M

    @property
    def lnq(self): return np.log(self.q)

    @q.setter
    def lnq(self, val): self.q = np.exp(val)

    @property
    def P(self):
        with np.errstate(invalid='ignore'):
            val = self.G*self.m*self.rho/(self.Gamma_1*self.r*self.A[:,2])

        val[self.x==0] = self.P_c
        return val

    @property
    def rho(self):
        with np.errstate(invalid='ignore'):
            val = self.A[:,5]*self.m/self.r**3/4./np.pi

        val[self.x==0] = self.rho_c
        return val

    @property
    def g(self): return self.G*self.m/self.r**2

    @property
    def Hp(self): return self.P/(self.rho*self.g)

    @property
    def Hrho(self): return 1/(1/self.Gamma_1/self.Hp + self.AA/self.r)

    @property
    def N2(self):
        val = self.AA*self.g/self.r
        val[self.x==0] = 0
        return val

    @property
    def cs2(self): return self.Gamma_1*self.P/self.rho

    @property
    def cs(self): return self.cs2**0.5

    # tau

class ADIPLSGrandSummary(object):
    """A class that represents the information for a set of mode
    frequencies, loaded from an ADIPLS grand summary file (often
    starting or ending with ``agsm``).  The main data is stored in the
    ``css`` attribute, which is a structured array.  This will usually
    be provided from a file by using `load_agsm` but an object can be
    constructed from any similarly structured array.

    A subset of the information in the ``css`` array is made available
    through attributes.

    Parameters
    ----------
    css: structured NumPy array
        The ``cs`` arrays for each mode.

    Attributes
    ----------
    M: float
        total mass
    R: float
        photospheric radius
    l: NumPy array of ints
        angular degrees
    n: NumPy array of ints
        angular degrees
    sigma2: NumPy array of floats
        square of the dimensionless angular eigenfrequency
    sigma2_c: NumPy array of floats
        square of the dimensionless angular eigenfrequency corrected
        for the Cowling approximation
    Pi_E: NumPy array of floats
        eigenperiod, in seconds
    Pi_V: NumPy array of floats
        variational period, in seconds
    nu_Ri: NumPy array of floats
        cyclic eigenfrequency corrected using Richardson
        extrapolation, in seconds
    nu_V: NumPy array of floats
        variational cyclic frequency, in Hz
    nu_E: NumPy array of floats
        cyclic eigenfrequency, in seconds
    nu_c: NumPy array of floats
        cyclic eigenfrequency corrected for the Cowling approximation,
        in seconds
    nu: NumPy array of floats
        alias of ``nu_c``
    E: NumPy array of floats
        Normalised mode inertia (see eq. (4.3) of ADIPLS notes).  Note
        that ADIPLS's definition is smaller than GYRE's by a factor of
        4π.
    beta: NumPy array of floats
        Weight for rotation kernel (see eq. (4.7) of ADIPLS notes or
        (8.43) of JCD's oscillation notes).

    """
    def __init__(self, css):
        self.css = css

    @property
    def M(self): return self.css[0]['M']

    @property
    def R(self): return self.css[0]['R']

    @property
    def l(self): return self.css['ell']

    @property
    def n(self): return self.css['enn']

    @property
    def sigma2(self): return self.css['sigma2']

    @property
    def sigma2_c(self): return self.css['sigma2_c']

    @property
    def Pi_E(self): return self.css['Pi_E']*60.0

    @property
    def Pi_V(self): return self.css['Pi_V']*60.0

    @property
    def nu_Ri(self): return self.css['nu_Ri']/1e3

    @property
    def nu_V(self): return self.css['nu_V']/1e3

    @property
    def nu_E(self): return 1/self.Pi_E

    @property
    def nu_c(self): return np.sqrt(self.sigma2_c/self.sigma2)/self.Pi_E

    @property
    def nu(self): return self.nu_c

    @property
    def E(self): return self.css['E']

    @property
    def beta(self): return self.css['beta_nl']

    def index_ln(self, l, n):
        """Returns the index of mode with angular degree *l* 
        and radial order *n*."""
        return np.where((self.l==l)&(self.n==n))[0][0]

    def index_nl(self, n, l):
        """Returns the index of mode with radial order *n* 
        and angular degree *l*."""
        return self.index_ln(l, n)


class ADIPLSEigenfunctions(ADIPLSGrandSummary):
    """A class that represents the information for a set of eigenfunction
    data kernels produced by ADIPLS.  This will usually be provided
    from a file by using `load_amde` but an object can be constructed
    from any similarly structured array.

    Parameters
    ----------
    css: structured NumPy array
        The ``cs`` arrays for each mode.
    eigs: 3-d NumPy array

        The eigenfunction arrays for each mode.  The *n*th element of
        the array has the eigenfunction data for the *n*th mode, in
        the same order as the summary data in *css*.  The number of
        rows in the array for a given mode is the number of meshpoints
        in the model.  The number of columns is either 6 or 2,
        depending on **nfmode**.
    nfmode: int
        The output mode used by ADIPLS' when the data was stored.
    x: NumPy array, optional
        If **nfmode** is 2 or 3, the fractional radius must be
        provided separately.  If **nfmode** is 1, it will be inferred
        from the eigenfunction data if not explicitly provided.

    This class has all the attributes of ``ADIPLSGrandSummary`` as
    well as the following extras.

    """
    def __init__(self, css, eigs, nfmode=1, x=None):
        ADIPLSGrandSummary.__init__(self, css)

        if nfmode == 1:
            if x is None:
                self.x = eigs[0,:,0]
            else:
                self.x = x
        elif nfmode == 2 or nfmode == 3:
            self.x = x
        else:
            raise ValueError('nfmode must be 1, 2 or 3, not %i' % nfmode)

        self.eigs = eigs


class ADIPLSRotationKernels(ADIPLSGrandSummary):
    """A class that represents the information for a set of rotational
    kernels produced by ADIPLS.  This will usually be provided from a
    file by using `load_rkr` but an object can be constructed from any
    similarly structured array.

    Parameters
    ----------
    css: structured NumPy array
        The ``cs`` arrays for each mode.
    rkrs: list of arrays
        The kernel arrays for each mode.  Each array has two columns:
        the fractional radius :math:`x` and the kernel :math:`K(x)`.

    This class has all the attributes of ``ADIPLSGrandSummary`` as
    well as the following extras.

    Attributes
    ----------
    x: NumPy array
        fractional radius co-ordinate
    K: list of NumPy arrays
        The *n*th row is the rotation kernel for the *n*th mode, in
        the same order as the summary data in *css*.  The mode with
        radial order *n* and angular degree *l* can be accessed by the
        functions ``K_ln(l,n)`` or ``K_nl(n,l)``.

    """
    def __init__(self, css, rkr):
        ADIPLSGrandSummary.__init__(self, css)
        self.x = rkr[0,:,0]
        self.K = rkr[:,:,1]

    def K_ln(self, l, n):
        "Load kernels by *l* and *n*."
        return self.K[(self.l==l)&(self.n==n)][0]

    def K_nl(self, n, l):
        "Load kernels by *n* and *l*."
        return self.K_ln(l, n)



from tomso import stars
import numpy as np
import unittest


class TestStarsFunctions(unittest.TestCase):

    def test_load_out(self):
        summaries, profiles = stars.load_out('data/stars.out')

        self.assertTrue(np.all(np.abs(profiles[0]['H1']-0.7) < 1e-9))
        self.assertTrue(np.all(np.abs(profiles[0]['He4']-0.28) < 1e-9))

        for profile in profiles:
            self.assertTrue(np.all(profile['k']), np.arange(1,200)[::-1])

        self.assertAlmostEqual(summaries[0]['dt'], 6308.266)

        for i, summary in enumerate(summaries[:10]):
            self.assertEqual(summary['n'], i+1)
            self.assertAlmostEqual(summary['H1_cntr'], 0.7)
            self.assertAlmostEqual(summary['He4_cntr'], 0.28)

    def test_load_plot(self):
        data = stars.load_plot('data/stars.plot')
        self.assertEqual(data['n'][0], 1)


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-

"""
Functions for manipulating `GYRE`_ input and output files.

.. _GYRE: https://bitbucket.org/rhdtownsend/gyre/wiki/Home
"""

import numpy as np
import warnings
from .common import DEFAULT_G, tomso_open, load_mesa_gyre


def load_summary(filename):
    """Reads a GYRE summary file and returns the global data and mode data
    in two structured arrays.  Uses builtin `gzip` module to read
    files ending with `.gz`.

    Parameters
    ----------
    filename: str
        Filename of the GYRE summary file to load.

    Returns
    -------
    header: structured array
        Global data for the frequency calculation. e.g. initial parameters.
        The keys for the array are the GYRE variable names as in
        the ``&output`` namelist in the GYRE input file.

    data: structured array
        Mode data for the frequency calculation. e.g. mode frequencies.
        The keys for the array are the GYRE variable names as in
        the ``&output`` namelist in the GYRE input file.

    """

    return load_mesa_gyre(filename, 'gyre')


def load_mode(filename):
    """Reads a GYRE mode file and returns the global data and mode profile
    data in two structured arrays.  Uses builtin `gzip` module to read
    files ending with `.gz`.

    Parameters
    ----------
    filename: str
        Filename of the GYRE mode file to load.

    Returns
    -------
    header: structured array
        Global data for the frequency calculation. e.g. initial parameters.
        The keys for the array are the GYRE variable names as in
        the ``&output`` namelist in the GYRE input file.

    data: structured array
        Mode data for the frequency calculation. e.g. mode frequencies.
        The keys for the array are the GYRE variable names as in
        the ``&output`` namelist in the GYRE input file.

    """

    return load_mesa_gyre(filename, 'gyre')


def load_gyre(filename, return_object=False):
    """Reads a GYRE stellar model file and returns the global data and
    point-wise data in a pair of NumPy record arrays.  Uses builtin
    `gzip` module to read files ending with `.gz`.

    If `return_object` is `True`, instead returns a `GYREStellarModel`
    object.  This will become default behaviour from v0.0.12.  The old
    behaviour will be dropped completely from v0.1.0.

    Parameters
    ----------
    filename: str
        Filename of the GYRE file.

    Returns
    -------
    header: structured array
        Global data for the stellar model. e.g. total mass, luminosity.

    data: structured array
        Profile data for the stellar model. e.g. radius, pressure.

    """
    def replace(s):
        # handles annoying Fortran formatting
        t = s[:]
        t = t.replace(b'D', b'E')
        t = t.replace(b'+', b'E+')
        t = t.replace(b'-', b'E-')
        t = t.replace(b'EE', b'E')
        t = t.replace(b' E-', b' -')
        return t

    with tomso_open(filename, 'rb') as f:
        lines = [replace(line) for line in f.readlines()]

    header_length = len(lines[0].split())
    if header_length == 4:
        version = 1
    elif header_length == 5:
        version = int(lines[0].split()[-1])
    else:
        raise ValueError("header should have 4 or 5 components but "
                         "it appears to have %i" % header_length)

    header = np.loadtxt(lines[:1], dtype=gyre_header_dtypes[version])
    data = np.loadtxt(lines[1:], dtype=gyre_data_dtypes[version])

    if return_object:
        return GYREStellarModel(header, data)
    else:
        warnings.warn("From tomso 0.1.0+, `gyre.load_gyre` will only return "
                      "a `GYREStellarModel` object: use `return_object=True` "
                      "to mimic future behaviour",
                      FutureWarning)
        return header, data


def save_gyre(filename, header, data):
    """Given the global data and point-wise data for a stellar model (as
    returned by :py:meth:`~tomso.gyre.load_gyre`), saves the data to a
    target file in the GYRE format.

    Parameters
    ----------
    filename: str
        Filename of the GYRE file.

    header: structured array
        Global data for the stellar model. e.g. total mass, luminosity.

    data: structured array
        Profile data for the stellar model. e.g. radius, pressure.

    """
    with open(filename, 'wt') as f:
        header_length = len(list(header[()]))
        # if header_length == 4:
        #     fmt = ''.join(['%6i','%26.16E'*3,'\n'])
        # elif header_length == 5:
        #     fmt = ''.join(['%6i','%26.16E'*3,'%6i\n'])
        # else:
        #     raise ValueError("header should have 4 or 5 components but "
        #                      "it appears to have %i" % header_length)
        if not 'version' in header.dtype.names:
            fmt = ''.join(['%6i','%26.16E'*3,'\n'])
        else:
            fmt = ''.join(['%6i','%26.16E'*3,'%6i\n'])

        f.writelines([fmt % tuple(header[()])])

        N = len(data[0])-1
        fmt = ''.join(['%6i',' %26.16E'*N,'\n'])
        for row in data:
            f.writelines([fmt % tuple(row)])


class GYREStellarModel(object):
    """A class that contains and allows one to manipulate the data stored
    a plain-text GYRE Stellar Model.

    The main attributes are the **header** and **data** record arrays,
    which store the data that's written in the text file.  The data in
    these arrays can be accessed via the attributes with more
    physically-meaningful names (e.g. the speed of sound is
    ``GYREStellarModel.cs``).

    Some of these values can also be set via the attributes if doing
    so is unambiguous. For example, the fractional radius **x** is not
    a member of the **data** array but setting **x** will assign
    the actual radius **r** to the corresponding values.  Values that
    are settable are indicated in the list of parameters.

    Parameters
    ----------
    header: structured array
        Global data for the stellar model. e.g. total mass, luminosity.

    data: structured array
        Profile data for the stellar model. e.g. radius, pressure.

    G: float, optional
        Value for the gravitational constant.  If not given (which is
        the default behaviour), we use the module-wise default value.

    Attributes
    ----------
    version: int
        file version number
    M: float, settable
        total mass
    R: float, settable
        photospheric radius
    L: float, settable
        total luminosity
    k: NumPy array
        mesh point number
    r: NumPy array, settable
        radius co-ordinate
    T: NumPy array, settable
        temperature
    P: NumPy array, settable
        pressure
    rho: NumPy array, settable
        density
    L_r: NumPy array, settable
        luminosity at radius **r**
    kappa: NumPy array, settable
        Rosseland mean opacity
    eps: NumPy array, settable
        specific energy generation rate
    Gamma_1: NumPy array
        first adiabatic index
    AA: NumPy array
        Ledoux discriminant
    x: NumPy array, settable
        fractional radius co-ordinate
    q: NumPy array, settable
        fractional mass co-ordinate
    m: NumPy array, settable
        mass co-ordinate
    w: NumPy array
        former fractional mass depth (w=m/(M-m))
    g: NumPy array
        local gravitational acceleration
    Hp: NumPy array
        pressure scale height
    Hrho: NumPy array
        density scale height
    N2: NumPy array
        squared Brunt–Väisälä (angular) frequency
    cs2: NumPy array
        squared adiabatic sound speed
    cs: NumPy array
        adiabatic sound speed
    U: NumPy array
        homology invariant *dlnm/dlnr*
    V: NumPy array
        homology invariant *dlnP/dlnr*
    Vg: NumPy array
        homology invariant *V/Gamma_1*

    """
    def __init__(self, header, data, G=DEFAULT_G):
        self.header = header
        self.data = data
        self.G = G

    def __len__(self):
        return len(self.data)

    def to_file(self, filename):
        """Save the model to a file.

        Parameters
        ----------
        filename: str
            Filename to which the data is written.
        """
        self.header['n'] = self.n
        save_gyre(filename, self.header, self.data)

    def to_fgong(self, reverse=True):
        """Convert the model to an ``FGONG`` object."""
        from .fgong import FGONG

        glob = np.zeros(15)
        glob[0] = self.M
        glob[1] = self.R
        glob[2] = self.L
        glob[14] = self.G

        var = np.zeros((len(self.data), 40))
        var[:,0] = self.r
        var[:,1] = self.lnq
        var[:,2] = self.T
        var[:,3] = self.P
        var[:,4] = self.rho
        var[:,9] = self.Gamma_1
        var[:,14] = self.AA

        if reverse:
            return FGONG(glob, var[::-1], ivers=300, G=self.G)
        else:
            return FGONG(glob, var, ivers=300, G=self.G)

    def to_amdl(self):
        """Convert the model to an ``ADIPLSStellarModel`` object."""
        from .adipls import ADIPLSStellarModel

        ioff = (0 if self.r[0] < 1e6 else 1) # mimic ADIPLS's FGONG to AMDL script
        A = np.zeros((len(self.data) + ioff, 6))

        # we can safely ignore division by 0 here
        with np.errstate(divide='ignore', invalid='ignore'):
            A[ioff:,0] = self.x
            A[ioff:,1] = self.q/self.x**3
            A[ioff:,2] = self.Vg
            A[ioff:,3] = self.Gamma_1
            A[ioff:,4] = self.AA
            A[ioff:,5] = self.U

        A[0,0] = 0.
        A[0,1] = 4.*np.pi/3.*self.rho[0]*self.R**3/self.M
        A[0,2] = 0.
        A[0,3] = self.Gamma_1[0]
        A[0,4] = 0.
        A[0,5] = 3.

        D = np.zeros(8)
        D[0] = self.M
        D[1] = self.R
        D[2] = self.P[0]
        D[3] = self.rho[0]
        D[4] = 4.*np.pi/3.*self.G*(self.rho[0]*self.R)**2/(self.P[0]*self.Gamma_1[0])
        D[5] = D[4]
        D[6] = -1.0
        D[7] = 0.0

        return ADIPLSStellarModel(D, A, G=self.G)

    @property
    def version(self):
        if 'version' in self.header.dtype.names:
            return self.header['version']
        else:
            return 1

    @property
    def n(self):
        return len(self.data)

    # Various properties for easier access to the data in `header` and
    # `data`.

    @property
    def M(self): return self.header['M']

    @M.setter
    def M(self, val): self.header['M'] = val

    @property
    def R(self): return self.header['R']

    @R.setter
    def R(self, val): self.header['R'] = val

    @property
    def L(self): return self.header['L']

    @L.setter
    def L(self, val): self.header['L'] = val

    @property
    def k(self): return self.data['k']

    @property
    def r(self): return self.data['r']

    @r.setter
    def r(self, val): self.data['r'] = val

    @property
    def L_r(self): return self.data['L_r']

    @L_r.setter
    def L_r(self, val): self.data['L_r'] = val

    @property
    def P(self): return self.data['P']

    @P.setter
    def P(self, val): self.data['P'] = val

    @property
    def T(self): return self.data['T']

    @T.setter
    def T(self, val): self.data['T'] = val

    @property
    def rho(self): return self.data['rho']

    @rho.setter
    def rho(self, val): self.data['rho'] = val

    @property
    def nabla(self): return self.data['nabla']

    @property
    def N2(self): return self.data['N2']

    @N2.setter
    def N2(self, val): self.data['N2'] = val

    @property
    def kappa(self): return self.data['kappa']

    @kappa.setter
    def kappa(self, val): self.data['kappa'] = val

    # Some properties have definitions that depend on the GYRE file
    # version.
    @property
    def w(self):
        if self.version in [1, 19]:
            return self.data['w']
        else:
            return self.data['m']/(self.header['M']-self.data['m'])

    @property
    def m(self):
        if self.version in [1, 19]:
            return self.data['w']*self.header['M']/(self.data['w']+1)
        else:
            return self.data['m']

    @m.setter
    def m(self, val):
        if self.version in [1, 19]:
            self.data['w'] = val/(self.M-w)
        else:
            self.data['m'] = val

    @property
    def Gamma_1(self):
        if self.version == 1:
            return self.data['c_P']/self.data['c_V']
        else:
            return self.data['Gamma_1']

    @Gamma_1.setter
    def Gamma_1(self, val):
        if self.version == 1:
            raise ValueError
        else:
            self.data['Gamma_1'] = val

    @property
    def eps(self):
        if self.version in [1, 19, 100]:
            return self.data['eps_tot']
        else:
            return self.data['eps']

    @property
    def Omega(self):
        if self.version == 1:
            return np.zeros_like(self.data['k'])
        else:
            return self.data['Omega']

    # Some convenient quantities derived from data in `header` and
    # `data` arrays.
    @property
    def x(self): return self.r/self.R

    @x.setter
    def x(self, val): self.r = val*self.R

    @property
    def q(self): return self.m/self.M

    @q.setter
    def q(self, val): self.m = val*self.M

    @property
    def lnq(self): return np.log(self.q)

    @lnq.setter
    def lnq(self, val): self.q = np.exp(val)

    @property
    def g(self):
        val = self.G*self.m/self.r**2
        val[self.r==0] = 0
        return val

    @property
    def AA(self):
        val = self.N2*self.r/self.g
        val[self.x==0] = 0
        return val

    @property
    def Hp(self): return self.P/(self.rho*self.g)

    @property
    def Hrho(self): return 1/(1/self.Gamma_1/self.Hp + self.AA/self.r)

    @property
    def cs2(self): return self.Gamma_1*self.P/self.rho

    @property
    def cs(self): return self.cs2**0.5

    @property
    def U(self):
        val = 4.*np.pi*self.rho*self.r**3/self.m
        val[self.r==0] = 3
        return val

    @property
    def V(self):
        val = self.G*self.m*self.rho/self.P/self.r
        val[self.r==0] = 0
        return val

    @property
    def Vg(self): return self.V/self.Gamma_1

gyre_header_dtypes = {1: [('n','int'), ('M','float'), ('R','float'),
                          ('L','float')],
                      19: [('n','int'), ('M','float'), ('R','float'),
                           ('L','float'), ('version','int')],
                      100: [('n','int'), ('M','float'), ('R','float'),
                            ('L','float'), ('version','int')],
                      101: [('n','int'), ('M','float'), ('R','float'),
                            ('L','float'), ('version','int')]}

gyre_data_dtypes = {1: [('k','int'), ('r','float'), ('w','float'),
                        ('L_r','float'), ('P','float'), ('T','float'),
                        ('rho','float'), ('nabla','float'),
                        ('N2','float'), ('c_V','float'), ('c_P','float'),
                        ('chi_T','float'), ('chi_rho','float'),
                        ('kappa','float'), ('kappa_T','float'),
                        ('kappa_rho','float'), ('eps_tot','float'),
                        ('eps_eps_T','float'), ('eps_eps_rho','float')],
                    19: [('k','int'), ('r','float'), ('w','float'),
                         ('L_r','float'), ('P','float'), ('T','float'),
                         ('rho','float'), ('nabla','float'),
                         ('N2','float'), ('Gamma_1','float'),
                         ('nabla_ad','float'), ('delta','float'),
                         ('kappa','float'), ('kappa_T','float'),
                         ('kappa_rho','float'), ('eps_tot','float'),
                         ('eps_eps_T','float'), ('eps_eps_rho','float'),
                         ('Omega','float')],
                    100: [('k','int'), ('r','float'), ('m','float'),
                          ('L_r','float'), ('P','float'), ('T','float'),
                          ('rho','float'), ('nabla','float'),
                          ('N2','float'), ('Gamma_1','float'),
                          ('nabla_ad','float'), ('delta','float'),
                          ('kappa','float'), ('kappa_kappa_T','float'),
                          ('kappa_kappa_rho','float'), ('eps_tot','float'),
                          ('eps_eps_T','float'), ('eps_eps_rho','float'),
                          ('Omega','float')],
                    101: [('k','int'), ('r','float'), ('m','float'),
                          ('L_r','float'), ('P','float'), ('T','float'),
                          ('rho','float'), ('nabla','float'),
                          ('N2','float'), ('Gamma_1','float'),
                          ('nabla_ad','float'), ('delta','float'),
                          ('kappa','float'), ('kappa_kappa_T','float'),
                          ('kappa_kappa_rho','float'), ('eps','float'),
                          ('eps_eps_T','float'), ('eps_eps_rho','float'),
                          ('Omega','float')]}



from tomso.adipls import load_amdl
from tomso.fgong import load_fgong
from tomso.gyre import load_gyre
from math import frexp
import unittest

# We should be able to make various round trips between different
# format and preserve the data relevant for adiabatic oscillations.
# Conversion to AMDL might modify the data because we mimic ADIPLS's
# own FGONG-to-AMDL script.

scalars = ['R', 'M', 'G']
vectors = ['r', 'q', 'm', 'rho', 'P', 'Gamma_1', 'N2', 'U', 'V']

class TestConversionFunctions(unittest.TestCase):
    def compare_floats(self, x, y, attr='', index=0, places=12):
        self.assertAlmostEqual(frexp(x)[0], frexp(y)[0], places=places,
                               msg='%s at k=%i' % (attr, index))

    def compare_models(self, m0, m1):
        self.assertEqual(len(m0), len(m1))

        for attr in scalars:
            self.compare_floats(getattr(m0, attr), getattr(m1, attr),
                                attr=attr)

        for attr in vectors:
            x0 = getattr(m0, attr)
            x1 = getattr(m1, attr)
            for i in range(len(m0)):
                self.compare_floats(x0[i], x1[i], attr=attr, index=i)

    def test_fgong_to_fgong(self):
        f = load_fgong('data/modelS.fgong', return_object=True)
        self.compare_models(f, f.to_gyre().to_fgong())

    def test_gyre_to_gyre(self):
        g = load_gyre('data/mesa.gyre', return_object=True)
        self.compare_models(g, g.to_fgong().to_gyre())

    def test_amdl_to_amdl(self):
        a = load_amdl('data/modelS.amdl', return_object=True)
        self.compare_models(a, a.to_fgong().to_amdl())
        self.compare_models(a, a.to_gyre().to_amdl())
        self.compare_models(a, a.to_fgong().to_gyre().to_amdl())
        self.compare_models(a, a.to_gyre().to_fgong().to_amdl())

if __name__ == '__main__':
    unittest.main()



from tomso import gyre
import numpy as np
import unittest

tmpfile = 'data/tmpfile'


class TestGYREFunctions(unittest.TestCase):

    def test_load_summary(self):
        header, data = gyre.load_summary('data/gyre.summ')
        self.assertAlmostEqual(header['M_star'], 1.989e33)
        self.assertAlmostEqual(header['R_star'], 6.959906258e10)
        for i, row in enumerate(data):
            self.assertEqual(row['l'], 1)
            self.assertEqual(row['n_pg'], i+19)

        header, data = gyre.load_summary('data/gyre_noheader.summ')
        for i, row in enumerate(data):
            self.assertEqual(row['l'], 0)
            self.assertEqual(row['n_pg'], i+8)

    def test_load_mode(self):
        for i in range(3):
            header, data = gyre.load_mode('data/gyre.mode_%i' % (i+1))
            self.assertEqual(header['n_pg'], i+19)
            self.assertEqual(header['l'], 1)
            self.assertEqual(header['Imomega'], 0.0)
            self.assertEqual(header['Imfreq'], 0.0)
            for row in data:
                self.assertEqual(row['Imxi_r'], 0.0)
                self.assertEqual(row['Imxi_h'], 0.0)

    def test_load_gyre(self):
        header, data = gyre.load_gyre('data/mesa.gyre')
        self.assertEqual(header['n'], 601)
        self.assertAlmostEqual(header['M'], 1.9882053999999999E+33)
        self.assertAlmostEqual(header['R'], 6.2045507132959908E+10)
        self.assertAlmostEqual(header['L'], 3.3408563666602257E+33)
        self.assertEqual(header['version'], 101)

        m = gyre.load_gyre('data/mesa.gyre', return_object=True)
        self.assertEqual(len(m.data), 601)
        self.assertAlmostEqual(m.M, 1.9882053999999999E+33)
        self.assertAlmostEqual(m.R, 6.2045507132959908E+10)
        self.assertAlmostEqual(m.L, 3.3408563666602257E+33)
        self.assertEqual(m.version, 101)

    def test_load_spb_mesa_versions(self):
        filenames = ['data/spb.mesa.78677cc', 'data/spb.mesa.813eed2',
                     'data/spb.mesa.adc6989']
        for filename in filenames:
            header1, data1 = gyre.load_gyre(filename)
            gyre.save_gyre(tmpfile, header1, data1)
            header2, data2 = gyre.load_gyre(tmpfile)
            self.assertEqual(header1, header2)
            for row1, row2 in zip(data1, data2):
                self.assertEqual(row1, row2)

            m1 = gyre.load_gyre('data/mesa.gyre', return_object=True)
            m1.to_file(tmpfile)
            m2 = gyre.load_gyre(tmpfile, return_object=True)
            self.assertEqual(m1.header, m2.header)
            for row1, row2 in zip(m1.data, m2.data):
                self.assertEqual(row1, row2)

            self.assertTrue(np.allclose(m1.r, m2.x*m2.R))
            self.assertTrue(np.allclose(m1.cs2, m2.Gamma_1*m2.P/m2.rho))
            self.assertTrue(np.allclose(m1.AA[1:], m2.N2[1:]*m2.r[1:]/m2.g[1:]))

    def test_save_gyre(self):
        header1, data1 = gyre.load_gyre('data/mesa.gyre')
        gyre.save_gyre(tmpfile, header1, data1)
        header2, data2 = gyre.load_gyre(tmpfile)
        self.assertEqual(header1, header2)
        for row1, row2 in zip(data1, data2):
            self.assertEqual(row1, row2)

        m1 = gyre.load_gyre('data/mesa.gyre', return_object=True)
        m1.to_file(tmpfile)
        m2 = gyre.load_gyre(tmpfile, return_object=True)
        self.assertEqual(m1.header, m2.header)
        for row1, row2 in zip(m1.data, m2.data):
            self.assertEqual(row1, row2)

        self.assertTrue(np.allclose(m1.r, m2.x*m2.R))
        self.assertTrue(np.allclose(m1.cs2, m2.Gamma_1*m2.P/m2.rho))
        self.assertTrue(np.allclose(m1.AA[1:], m2.N2[1:]*m2.r[1:]/m2.g[1:]))


if __name__ == '__main__':
    unittest.main()



from setuptools import setup

setup(
    name = 'tomso',
    packages = ['tomso'],
    version = '0.0.11',
    description = 'Tools for Modelling Stars and their Oscillations',
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    author = 'Warrick Ball',
    author_email = 'W.H.Ball@bham.ac.uk',
    url = 'https://github.com/warrickball/tomso',
    download_url = 'https://github.com/warrickball/tomso/archive/v0.0.11.tar.gz',
    install_requires=['numpy'],
    keywords = [],
    scripts = ['scripts/tomso-convert'],
    license = 'MIT',
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python"
        ]
    )
